/**********************************************************************
 * File  : game_menus.cpp
 * Author: Lightness1024
 *         mailto:lightness1024@free.fr
 * Date  : 05/05/2009
 *********************************************************************/

#include "randclass.hpp"
#include "myutils.hpp"

#include "spritemanager.hpp"

#include "terraingenerator.hpp"
#include "carsmanager.hpp"
#include "hostilesmanager.hpp"
#include "geometry.hpp"
#include "weaponmanager.hpp"
#include "towermanager.hpp"
#include "hud.hpp"
#include "collisionmanager.hpp"
#include "proximity_manager.hpp"
#include "supramanager.hpp"
#include "highscore.hpp"

#include "sdl_events.hpp"

#include "game_menus.hpp"

namespace tips
{
	boost::array< char const*, 3 > const  bar = {{"resources/hud/tips_ctrl_bar_scroll.bmp",
												  "resources/hud/tips_end_bar_scroll.bmp",
												  "resources/hud/tips_home_bar_scroll.bmp"}};
}

void BuyItem(int menuPosId)
{
	game_vars::shop::g_pmenu->SetCursorPosition(menuPosId);

	bool accepted = false;
	if (menuPosId == game_vars::shop::g_ccd_id)	// achat du control device
	{
		if (game_vars::g_user->score >= 3000 && !game_vars::g_cheat)
		{
			game_vars::g_carControlDevice = true;
			game_vars::g_user->score -= 3000;
			ChiefManager::GetSoundManager()->ReadSound("repair");
			accepted = true;
		}
	}
	else if (menuPosId == game_vars::shop::g_shield_id)  // achat de bouclier
	{
		const int& shldv = game_vars::shop::shield_value;
		int max_shieldage = game_vars::g_cheat ? 2000 : 3;  // on a le droit de cumuler 3 boucliers
		if (game_vars::g_user->score >= 700 && game_vars::g_user->shield < max_shieldage * shldv)
		{
			game_vars::g_user->score  -= 700;
			game_vars::g_user->shield += shldv;
			ChiefManager::GetSoundManager()->ReadSound("repair");
			accepted = true;
		}
	}
	else if (menuPosId > 0)	 // selection arme
	{
		// pour faire parfaitement joli il faudrait mapper
		// les id du menu aux id des armes. cependant vu que le menu et les armes sont construits dans le meme ordre,
		// les id correspondent.
		if (game_vars::g_user->WeaponPresent(menuPosId))	// on a déjà l'arme
		{
			game_vars::g_currentWeapon = menuPosId;
			ChiefManager::GetSoundManager()->ReadSound("loaded");
			accepted = true;
		}
		else	// l'user essaye de l'acheter
		{
			// s'il a l'argent ou si il triche :
			if (game_vars::g_user->score >= ChiefManager::GetWeaponManager()->GetPrice(menuPosId) || game_vars::g_cheat)
			{
				// on débite ca de son compte
				game_vars::g_user->score -= ChiefManager::GetWeaponManager()->GetPrice(menuPosId);
				// et on lui file l'arme:
				game_vars::g_user->GiveWeapon(menuPosId);
				game_vars::g_currentWeapon = menuPosId;
				if (game_vars::g_currentWeapon == 4)	// wouw quadruple SFG :)
					ChiefManager::GetSoundManager()->ReadSound("powerful");
				else
					ChiefManager::GetSoundManager()->ReadSound("loaded");
				accepted = true;
			}
		}
	}
	else	// selection tourelle
	{
		menuPosId = -menuPosId;
		if (game_vars::g_user->score >= ChiefManager::GetTowerManager()->GetPrice(menuPosId) || game_vars::g_cheat) // pareil
		{
			// on essaie de construire la tourelle:
			Vector2i pos = game_vars::g_user->rect.GetCenter();
			pos -= (game_vars::g_user->direction * 58.f);
			if (ChiefManager::GetTowerManager()->BuildNewTower(pos.x, pos.y, menuPosId, &(*game_vars::g_user)))
			{
				game_vars::g_user->score -= ChiefManager::GetTowerManager()->GetPrice(menuPosId);	// si on a pu la construire ici, on debite.
				accepted = true;
			}
		}
	}

	if (!accepted)
		ChiefManager::GetSoundManager()->ReadSound("beeep");

	game_vars::shop::g_menuActivated = false;
}


EMainMenuCommands WelcomeMenu(void)
{
	// image de fond:
	SpriteManager sp("extreme carnage menu");
	sp.SetScrollSensible(false);
	sp.LoadSprite("resources/splash.bmp", SP_STATIC);

	// menu :
	Menu wm;

	wm.SetLeftStartPos(GetScreenCenterX(128) - 20, 100);
	// en tete:
	wm.SetHeadLeft(GetScreenCenterX(256), 20, "resources/hud/main_menu_head.bmp");

	// elements du menu
	wm.PushLeft("resources/hud/main_menu_start_reference.bmp");
	wm.PushLeft("resources/hud/main_menu_start_custom.bmp");
	wm.PushLeft("resources/hud/main_menu_scores.bmp");
	wm.PushLeft("resources/hud/main_menu_sound.bmp");
	wm.PushLeft("resources/hud/main_menu_quit.bmp");
	// curseur:
	wm.SetArrow("resources/hud/arrow.bmp");

	EKeyboardMenuEntries key = KBME_NONE;
	while (KBME_EXIT != key)
	{
		sp.Draw(0);
		wm.Draw();
		sp.Update();

		key = HandleMenuEvent();
		if (key == KBME_UP)
			wm.SignalCursorMove(M_CUR_RIGHT);
		else if (key == KBME_DOWN)
			wm.SignalCursorMove(M_CUR_LEFT);
		else if (key == KBME_VALIDATE)
		{
			switch (wm.GetCursorPosition())
			{
				case  0: return MC_START;
				case -1: return MC_CUSTOMIZED_TERRAIN;
				case -2: return MC_HIGHSCORES;
				case -3: return MC_SOUND_SETTINGS;
				case -4: return MC_EXITGAME;
			}
		}
	}
	return MC_EXITGAME;
}

void PlayMenuOpeningSound(void)
{
	static int menuopenings = 0;

	const char* sndName;
	menuopenings++ % 5 == 0 ? sndName = "openmenu1" : sndName = "openmenu2";

	ChiefManager::GetSoundManager()->ReadSound(sndName);
}

void HighScoresMenu(void)
{
	SpriteManager spm("extreme carnage highscores");
	spm.SetScrollSensible(false);
	Hud hud;

	int bg = spm.LoadSprite("resources/splash_dark.bmp", SP_STATIC);

	int fontid = hud.PushNewNumbersFontSprite("resources/hud/numbers.bmp");

	// charger icones des joueurs
	int pid[2] = {spm.LoadSprite(MakeBmpFN(GetPlayerFromIndex(0)).c_str(), SP_STATIC),
				  spm.LoadSprite(MakeBmpFN(GetPlayerFromIndex(1)).c_str(), SP_STATIC)};

	int cpid = spm.LoadSprite("resources/hud/cp.bmp", SP_STATIC);
	int any = spm.LoadSprite("resources/hud/compiling_anykey.bmp", SP_STATIC);

	// dessiner le background avant:
	spm.Draw(bg, 0, 0);

	// afficher les sprites et les scores des joueurs dans l'ordre de leur scores
	HighscoreManager hsm;
	struct HighScores_File_Corrupted {};
	for (size_t i = 0; i < std::min<size_t>(hsm.GetScoreNum(), 6); ++i)  // pour chaque score
	{
		ScoreRecord rec = hsm.GetScoreRec(i);

		if (GetIndexOfPlayer(rec.playerName) == -1)
			throw HighScores_File_Corrupted();

		int const y = 70 + i * 80;
		int const x = 270;

		// dessiner icone du joueur :
		spm.Draw(pid[GetIndexOfPlayer(rec.playerName)], x, y - 18);

		// nombre de points du joueur :
		int objid = hud.PushNewHudElement(x + 110, y, NULL, fontid);
		hud.UpdateNumberValue(objid, rec.score);

		// petit icone CP devant
		spm.Draw(cpid, x + 70, y);

		// et indication "press any key" en bas:
		spm.Draw(any, GetScreenCenterX(spm.GetW(any)), 560);
	}
	hud.Draw();
	spm.Update();

	EKeyboardMenuEntries key = KBME_NONE;
	while (key == KBME_NONE || key >= KBME_KUP_UNHANDLED)
	{
		key = HandleMenuEvent();
	}
}

SoundSettings SoundSettingsMenu(int startMusicVolume, SoundManager* menuMusic)
{
	SpriteManager spm("extreme carnage - sound settings");
	spm.SetScrollSensible(false);
	Hud hud;

	int bg = spm.LoadSprite("resources/splash_dark.bmp", SP_STATIC);
	int mv = spm.LoadSprite("resources/hud/music_volume.bmp", SP_STATIC);
	int accept = spm.LoadSprite("resources/hud/accept_snd_settings.bmp", SP_STATIC);
	int tip = spm.LoadSprite(tips::bar[time(0) % 3], SP_STATIC);

	int fontid = hud.PushNewNumbersFontSprite("resources/hud/numbers_small.bmp");

	int numid = hud.PushNewHudElement(570, 205, 0, fontid);

	int barid = hud.PushProgressBar(320,
									200,
									"resources/hud/adjust_bar_blue.bmp",
									"resources/hud/adjust_bar_darkblue.bmp",
									Hud::PB_WITH_ACTIVATED_PARTS);

	EKeyboardMenuEntries key = KBME_NONE;
	int musicVolume = startMusicVolume;
	int barSpeed = 0;
	int accelerator = 1;  // ceci peut être accéléré par la pression de la touche contrôle
	while (KBME_EXIT != key && KBME_VALIDATE != key)
	{
		musicVolume += barSpeed * accelerator;
		musicVolume = myutils::clamp(musicVolume, 0, 100);

		hud.UpdateNumberValue(numid, musicVolume);

		hud.UpdateProgressBar(barid, musicVolume);
		spm.Draw(bg);
		spm.Draw(mv, 180, 197);
		spm.Draw(accept, GetScreenCenterX(spm.GetW(accept)), 80);
		spm.Draw(tip, GetScreenCenterX(spm.GetW(tip)), 550);
		hud.Draw();
		spm.Update();

		menuMusic->SetMusicVolume(musicVolume);

		key = HandleMenuEvent(30);
		switch (key)
		{
			case KBME_RIGHT:
			case KBME_UP:
				barSpeed = 1;
				break;
			case KBME_LEFT:
			case KBME_DOWN:
				barSpeed = -1;
				break;
			case KBME_KUP_RIGHT:
			case KBME_KUP_UP:
			case KBME_KUP_LEFT:
			case KBME_KUP_DOWN:
				barSpeed = 0;
				break;
			case KBME_END:
				musicVolume = 100;
				break;
			case KBME_HOME:
				musicVolume = 0;
				break;
			case KBME_CTRL:
				accelerator = 5;
				break;
			case KBME_KUP_CTRL:
				accelerator = 1;
				break;
			default:
				break;
		}
	}
	SoundSettings settings;
	if (key == KBME_VALIDATE)
	{
		settings.musicVolume = musicVolume;
	}
	else
	{
		menuMusic->SetMusicVolume(startMusicVolume);
		settings.musicVolume = -1;
	}
	return settings;
}

void DisplayCreditsScreen(void)
{
	SpriteManager sp("extreme carnage credits");
	sp.SetScrollSensible(false);
	sp.ClearScreen();
	int spid = sp.LoadSprite("resources/hud/credits.bmp", SP_STATIC);
	sp.Move(spid, GetScreenCenterX(sp.GetW(spid)), GetScreenCenterY(sp.GetH(spid)));

	for (;;)
	{
		sp.Update();
		sp.Draw(0);

		SDL_Event event;
		if (SDL_WaitEvent(&event))
		{
			switch (event.type)
			{
				case SDL_QUIT:
					return;
				case SDL_KEYDOWN:
					return;
			}
		}
	}
}

EChoosePlayerMenuCommands ChoosePlayerMenu(void)
{
	SpriteManager spm("extremecarnage - choose player");
	spm.SetScrollSensible(false);
	spm.LoadSprite("resources/splash_dark.bmp", SP_STATIC);

	// petit menu
	Menu m;

	m.SetLeftStartPos(GetScreenCenterX(160), 240);
	// en tete:
	m.SetHeadLeft(GetScreenCenterX(256), 140, "resources/hud/player_menu_head.bmp");

	// joueurs
	m.PushLeft("resources/hud/players/badguy.bmp");
	m.PushLeft("resources/hud/players/morph.bmp");
	m.PushLeft("resources/hud/player_menu_return.bmp");
	// curseur:
	m.SetArrow("resources/hud/arrow.bmp");

	EKeyboardMenuEntries key = KBME_NONE;
	while (KBME_EXIT != key)
	{
		spm.DrawAll();
		m.Draw();
		spm.Update();

		key = HandleMenuEvent();
		if (key == KBME_UP)
			m.SignalCursorMove(M_CUR_RIGHT);
		else if (key == KBME_DOWN)
			m.SignalCursorMove(M_CUR_LEFT);
		else if (key == KBME_VALIDATE)
		{
			switch (m.GetCursorPosition())
			{
				case 0:
					return CPMC_BAD_GUY;
				case -1:
					return CPMC_MORPH;
				case -2:
					return CPMC_RETURN;
			}
		}
	}
	return CPMC_RETURN;
}

EPauseMenuChoice PauseMenu(void)
{
	SpriteManager spm("extremecarnage - pause menu");
	spm.SetScrollSensible(false);
	int bg = spm.LoadSprite("resources/hud/pm_bg.bmp", SP_STATIC);

	// petit menu
	Menu m;

	m.SetLeftStartPos(GetScreenCenterX(128) - 12, GetScreenCenterY(spm.GetH(bg)) + 100);

	// choix
	m.PushLeft("resources/hud/pm_continue.bmp");
	m.PushLeft("resources/hud/pm_quit.bmp");
	// curseur:
	m.SetArrow("resources/hud/arrow.bmp");

	EKeyboardMenuEntries key = KBME_NONE;
	while (KBME_EXIT != key)
	{
		spm.Draw(bg, GetScreenCenterX(spm.GetW(bg)), GetScreenCenterY(spm.GetH(bg)));
		m.Draw();
		spm.Update();

		key = HandleMenuEvent();
		if (key == KBME_UP)
			m.SignalCursorMove(M_CUR_RIGHT);
		else if (key == KBME_DOWN)
			m.SignalCursorMove(M_CUR_LEFT);
		else if (key == KBME_VALIDATE)
		{
			switch (m.GetCursorPosition())
			{
				case  0: return PMC_CONTINUE;
				case -1: return PMC_QUIT;
			}
		}
	}
	return PMC_QUIT;
}

void DrawShopMenu(Menu& shop_menu, SpriteManager& spm)
{
	shop_menu.Draw(Menu::NO_FLIP);
	// dessiner les touches associées aux elements
	game_vars::shop::NumToIDs::const_iterator it = game_vars::shop::g_key_to_id.begin();
	while (it != game_vars::shop::g_key_to_id.end())
	{
		Vector2i p = shop_menu.GetElemPos(it->second.menuid);
		Vector2i s = shop_menu.GetElemSize(it->second.menuid);
		spm.Draw(it->second.spriteid, p.x + s.x, p.y - s.y / 3, 0);
		++it;
	}
}

// menu de choix des parametres du terrain
std::pair< CustomTerrainParams, bool > CustomizeTerrainMenu(CustomTerrainParams const& in_values)
{
	SpriteManager spm("extreme carnage - procedural terrain parameters");
	spm.SetScrollSensible(false);
	Hud hud;

	int bg = spm.LoadSprite("resources/splash_dark.bmp", SP_STATIC);
	int head = spm.LoadSprite("resources/hud/custom_param_menu_title.bmp", SP_STATIC);
	int warn = spm.LoadSprite("resources/hud/custom_param_warning.bmp", SP_STATIC);
	int tip = spm.LoadSprite(tips::bar[time(0) % 3], SP_STATIC);

	int arrow = spm.LoadSprite("resources/hud/arrow.bmp", SP_ANIMATED);

	int fontid = hud.PushNewNumbersFontSprite("resources/hud/numbers_small.bmp");

	int const firstY = 110;
	int const spacingY = 50;
	char const* fnames[] = {"resources/hud/custom_param_water.bmp",
							"resources/hud/custom_param_rock.bmp",
							"resources/hud/custom_param_tree.bmp",
							"resources/hud/custom_param_size.bmp",
							"resources/hud/custom_param_en_flow.bmp",
							"resources/hud/custom_param_seed.bmp"};

	BOOST_ASSERT( sizeof(fnames) / sizeof(fnames[0]) == CTMO_NUM_OBJ );  // STATIC_ASSERT irait mieux...
	BOOST_ASSERT( (int)KTO_WATER == (int)CTMO_WATER );
	BOOST_ASSERT( (int)KTO_ROCKS == (int)CTMO_ROCKS );
	BOOST_ASSERT( (int)KTO_TREE == (int)CTMO_TREE );

	boost::array< NumeredProgressBar, CTMO_NUM_OBJ > prbars;

	for (size_t i = 0; i < prbars.size(); ++i)
	{
		int y = i * spacingY + firstY;
		prbars[i].prbarid = hud.PushProgressBar(204,
												y,
												"resources/hud/adjust_bar_blue.bmp",
												"resources/hud/adjust_bar_darkblue.bmp",
												Hud::PB_WITH_ACTIVATED_PARTS);

		prbars[i].hudfontid = hud.PushNewHudElement(448, y, 0, fontid);
		switch (i)
		{
			case CTMO_WATER:
			case CTMO_ROCKS:
			case CTMO_TREE: prbars[i].value = in_values.density[i] * 1000.f; break;  // le warning ici est un bug de gcc.
			case CTMO_SIZE: prbars[i].value = in_values.size_long / 5.f; break;
			case CTMO_FLOW: prbars[i].value = in_values.flow / 8; break;
			case CTMO_SEED: prbars[i].value = in_values.seed; break;
		}

		prbars[i].titleid = spm.LoadSprite(fnames[i], SP_STATIC);
	}

	EKeyboardMenuEntries key = KBME_NONE;
	size_t activeBar = 0;
	int speed = 1;
	int accelerator = 0;
	while (KBME_EXIT != key && KBME_VALIDATE != key)
	{
		spm.Draw(bg);
		for (size_t i = 0; i < prbars.size(); ++i)
		{
			hud.UpdateNumberValue(prbars[i].hudfontid, prbars[i].value);
			hud.UpdateProgressBar(prbars[i].prbarid, prbars[i].value);
			spm.Draw(prbars[i].titleid, 478, firstY + spacingY * i - 5);
		}
		spm.Draw(head, GetScreenCenterX(spm.GetW(head)), 22);
		spm.Draw(warn, GetScreenCenterX(spm.GetW(warn)), 400);
		spm.Draw(tip, GetScreenCenterX(spm.GetW(tip)), 550);
		spm.Draw(arrow, 144, activeBar * spacingY + firstY);
		hud.Draw();
		spm.Update();

		key = HandleMenuEvent(30);
		switch (key)
		{
			case KBME_KUP_LEFT:
			case KBME_KUP_RIGHT:
				accelerator = 0;
				break;
			case KBME_RIGHT:
				accelerator = 1;
				break;
			case KBME_LEFT:
				accelerator = -1;
				break;
			case KBME_UP:
				activeBar = activeBar > 0 ? activeBar - 1 : 0;
				break;
			case KBME_DOWN:
				activeBar = activeBar < prbars.size() - 1 ? activeBar + 1 : activeBar;
				break;
			case KBME_END:
				prbars[activeBar].value = 100;
				break;
			case KBME_HOME:
				prbars[activeBar].value = 0;
				break;
			case KBME_CTRL:
				speed = 5;
				break;
			case KBME_KUP_CTRL:
				speed = 1;
				break;
			default:
				break;
		}
		int lowbound = (activeBar == CTMO_SIZE) ? 3 : 0;
		prbars[activeBar].ChangeValueClamped(accelerator * speed, lowbound, 100);
	}
	CustomTerrainParams out;
	for (size_t i = 0; i < CTMO_NUM_OBJ; ++i)
	{
		switch (i)
		{
			case CTMO_WATER:
			case CTMO_ROCKS:
			case CTMO_TREE: out.density[i] = prbars[i].value * 0.001f; break;
			case CTMO_SIZE: out.size_long = prbars[i].value * 5.f; break;
			case CTMO_FLOW: out.flow = prbars[i].value * 8; break;
			case CTMO_SEED: out.seed = prbars[i].value; break;
		}
	}
	std::pair< CustomTerrainParams, bool > result(out, KBME_VALIDATE == key);
	return result;
}

int FinalScoreMenu(int cash_left, int cumulated_cp, bool fast_animation)
{
	SpriteManager spm("extreme carnage - final score");
	spm.SetScrollSensible(false);

	spm.ClearScreen();

	int title = spm.LoadSprite("resources/hud/compiling_score.bmp", SP_STATIC);
	spm.Move(title, GetScreenCenterX(spm.GetW(title)), 32);
	int cashleft = spm.LoadSprite("resources/hud/compiling_cashleft.bmp", SP_STATIC);
	spm.Move(cashleft, GetScreenCenterX(0) - 240, 110);
	int cumulated = spm.LoadSprite("resources/hud/compiling_cumulated.bmp", SP_STATIC);
	spm.Move(cumulated, GetScreenCenterX(0) - 240, 150);
	int final = spm.LoadSprite("resources/hud/compiling_finalscore.bmp", SP_STATIC);
	spm.Move(final, GetScreenCenterX(0) - 260, 200);
	int bg = spm.LoadSprite("resources/hud/compiling_bg.bmp", SP_STATIC);
	spm.Move(bg, GetScreenCenterX(0) - 10, 100);
	int cp = spm.LoadSprite("resources/hud/cp.bmp", SP_STATIC);
	spm.Move(cp, GetScreenCenterX(0) + 140, 196);

	Hud hud;
	int fontid = hud.PushNewNumbersFontSprite("resources/hud/numbers.bmp");
	int cashnum = hud.PushNewHudElement(GetScreenCenterX(0), 112, NULL, fontid);
	int cumulnum = hud.PushNewHudElement(GetScreenCenterX(0), 152, NULL, fontid);
	int finalnum = hud.PushNewHudElement(GetScreenCenterX(0), 202, NULL, fontid);

	// faire un tite animation de transfert:

	tools::HiResTimer tmr;

	int final_score = 0;
	bool firststep = true;
	// pts par secondes:
	int speed = fast_animation ? 500000 : 100000;

	tmr.Start();
	double last = tmr.GetTime();
	double error = 0.;
	bool ignore_next_evnt = false;
	FlushEvents();
	while (cash_left + cumulated_cp > 0)
	{
		if (!firststep)  // la premiere fois on veut afficher le tout sans alteration
		{
			double delta_time = tmr.GetTime() - last;
			last = tmr.GetTime();

			double dstep = speed * delta_time;
			int step = dstep;
			error += dstep - step;
			int ierror = error;
			step -= error;
			error -= ierror;

			int diff_cash = cash_left - step > 0 ? step : cash_left;
			int diff_cumu = cumulated_cp - step > 0 ? step : cumulated_cp;

			cash_left -= diff_cash;
			cumulated_cp -= diff_cumu;
			final_score += diff_cumu + diff_cash;
		}

		// afficher le tout :
		hud.UpdateNumberValue(cashnum, cash_left);
		hud.UpdateNumberValue(cumulnum, cumulated_cp);
		hud.UpdateNumberValue(finalnum, final_score);
		spm.DrawAll();
		hud.Draw();
		spm.Update();

		if (firststep)  // avant de faire défiler les scores, on attend une seconde pour le "show off" :)
		{
			firststep = false;
			tmr.Stop();
			int msOfWait = fast_animation ? 300 : 1000;
			EKeyboardMenuEntries ret = HandleMenuEvent(msOfWait);
			if (ret != KBME_NONE && ret < KBME_KUP_UNHANDLED)
			{
				ignore_next_evnt = true;   // parce que souvent le prochain evenement est juste le relachement de la touche qui a fait sortir de l'attente
			}
			tmr.Start();
		}
		if (EventPending() && ignore_next_evnt)
		{
			FlushEvents();
			ignore_next_evnt = false;
		}
		if (EventPending() && !ignore_next_evnt)
		{
			speed = 1e6;
		}
	}

	int any = spm.LoadSprite("resources/hud/compiling_anykey.bmp", SP_STATIC);
	spm.Move(any, GetScreenCenterX(spm.GetW(any)), 280);

	spm.DrawAll();
	hud.Draw();
	spm.Update();

	FlushEvents();
	HandleMenuEvent(100000);
	return final_score;
}

void KeySummaryScreen(void)
{
	SpriteManager spm("extreme carnage - keys summary - ready to go ?");
	spm.SetScrollSensible(false);

	spm.ClearScreen();
	
	int help = spm.LoadSprite("resources/key_help.bmp", SP_STATIC);
	spm.Move(help, GetScreenCenterX(spm.GetW(help)), GetScreenCenterY(spm.GetH(help)));
	spm.DrawAll();
	spm.Update();
	
	tools::HiResTimer::Sleep(500);
	FlushEvents();
	HandleMenuEvent(100000);
}
