/**********************************************************************
 * File  : iaclasses.hpp
 * Author: Lightness1024
 *         mailto:lightness1024@free.fr
 * Date  : 24/03/2009
 *********************************************************************/

#ifndef IACLASSES_HPP_INC_24_03_2009__170461501_
#define IACLASSES_HPP_INC_24_03_2009__170461501_ 1

#include <map>
#include <string>
#include "array.hpp"
#include "noncopyable.hpp"

namespace ia
{
	enum ERelationType
	{
		FRIEND,
		NEUTRA,
		ENNEMY
	};

	// pour pouvoir stocker des pointeurs vers n'importe quelle "instance du template" d'un Manager
	struct ManagerBase
	{
		virtual ERelationType GetRelation(char const * class1, char const * class2) const = 0;
		virtual ERelationType GetRelation(size_t id_class1, size_t id_class2) const = 0;
		virtual size_t GetClassID(char const * name) const = 0;
		virtual std::string const & GetClassName(size_t classID) const = 0;
		virtual bool ClassExists(char const * name) const = 0;

		virtual ~ManagerBase() {};
	};

	template< size_t N_CLASSES >
	class Manager : boost::noncopyable, public ManagerBase
	{
		typedef boost::array< ERelationType, N_CLASSES > MatrixLine;
		typedef boost::array< MatrixLine, N_CLASSES > RelationMatrixImpl;

		RelationMatrixImpl relations_;

		// informations des classes
		struct IAClasses
		{
			typedef std::map< std::string, size_t > NameMap;  // relie les noms aux positions dans la matrice
			typedef NameMap::const_iterator   NameMapCtIter;
			typedef std::vector< NameMapCtIter > ReverseMap;  // relie les positions dans la matrice aux noms
			
			NameMap  names;
			ReverseMap ids;
			
			size_t GetID(char const * name) const;
			std::string const & GetName(size_t id) const;
		
		} classes_;

	public:

		typedef ERelationType RelMatrix[N_CLASSES][N_CLASSES];
		typedef char const * StringsRow[N_CLASSES];

		Manager();

		void SetIAClassNames(StringsRow const & names);

		void SetRelations(RelMatrix const & matrix);
		void SetRelation(char const * class1, char const * class2, ERelationType newRelation);

		ERelationType GetRelation(char const * class1, char const * class2) const;
		ERelationType GetRelation(size_t id_class1, size_t id_class2) const;
		size_t GetClassID(char const * name) const;
		std::string const & GetClassName(size_t classID) const;
		bool ClassExists(char const * name) const;
	};

	class Class
	{
		std::string iaClass_;
		size_t iaID_;

	public:

		Class() : iaClass_("NOCLASS (uninitialized)"),
				  iaID_(123456789)
		{}

		std::string const & GetIAClassName(void) const
		{
			return iaClass_;
		}

		size_t GetIAClassID(void) const
		{
			return iaID_;
		}

		void SetIAClass(size_t classID, ManagerBase const * mgr)
		{
			iaID_ = classID;
			iaClass_ = mgr->GetClassName(classID);
		}

		void SetIAClass(char const * side, ManagerBase const * mgr)
		{
			iaClass_ = side;
			iaID_ = mgr->GetClassID(side);
		}

		ERelationType RelationWith(Class const & other, ManagerBase const * mgr) const
		{
			return mgr->GetRelation(iaClass_.c_str(), other.iaClass_.c_str());
		}
	};
}

#include "iaclasses.inl"

#endif //IACLASSES_HPP_INC_24_03_2009__170461501_
