/**********************************************************************
 * File  : iaclasses.cpp
 * Author: Lightness1024
 *         mailto:lightness1024@free.fr
 * Date  : 24/03/2009
 *********************************************************************/

#include "iaclasses.hpp"
#include <exception>
#include <algorithm>

namespace ia
{
	template< size_t N >
	Manager<N>::Manager()
	{
		for (size_t x = 0; x < N; ++x)
		{
			for (size_t y = 0; y < N; ++y)
			{
				relations_[x][y] = NEUTRA;
			}
		}
	}

	template< size_t N >
	void Manager<N>::SetIAClassNames(StringsRow const & names)
	{
		classes_.ids.resize(N);
		for (size_t i = 0; i < N; ++i)
		{
			classes_.names[names[i]] = i;
			classes_.ids[i] = classes_.names.find(names[i]);
		}
	}

	template< size_t N >
	void Manager<N>::SetRelations(RelMatrix const & matrix)
	{
		for (size_t x = 0; x < N; ++x)
		{
			for (size_t y = 0; y < N; ++y)
			{
				relations_[x][y] = matrix[x][y];
				if (relations_[x][y] < FRIEND || relations_[x][y] > ENNEMY)
					throw std::invalid_argument("IA Manager is fed with an invalid relation matrix");
			}
		}
	}

	template< size_t N >
	void Manager<N>::SetRelation(char const * class1, char const * class2, ERelationType newRelation)
	{
		int x = classes_.GetID(class1);
		int y = classes_.GetID(class2);
		relations_[x][y] = newRelation;
	}

	template< size_t N >
	ERelationType Manager<N>::GetRelation(char const * class1, char const * class2) const
	{
		int x = classes_.GetID(class1);
		int y = classes_.GetID(class2);
		return relations_[x][y];
	}

	template< size_t N >
	ERelationType Manager<N>::GetRelation(size_t id_class1, size_t id_class2) const
	{
		return relations_[id_class1][id_class2];
	}

	template< size_t N >
	bool Manager<N>::ClassExists(char const * name) const
	{
		return classes_.names.find(name) != classes_.names.end();
	}

	template< size_t N >
	size_t Manager<N>::GetClassID(char const * name) const
	{
		return classes_.GetID(name);
	}

	template< size_t N >
	std::string const & Manager<N>::GetClassName(size_t classID) const
	{
		return classes_.GetName(classID);
	}

	// private
	template< size_t N >
	size_t Manager<N>::IAClasses::GetID(char const * name) const
	{
		NameMapCtIter it = names.find(name);
		if (it == names.end())
		{
			throw std::invalid_argument("IA Manager : trying to access a class that doesn't exist");
		}
		return it->second;
	}
	// private
	template< size_t N >
	std::string const & Manager<N>::IAClasses::GetName(size_t id) const
	{
		if (id >= N)
		{
			throw std::invalid_argument("IA Manager : trying to access a class that doesn't exist");
		}
		return ids[id]->first;
	}	
}
