/**********************************************************************
 * File	: carsmanager.cpp
 * Author: Vivien - a.k.a. Lightness1024!
 *			  mailto:Lightness1024@free.fr
 * Date	: 12/07/2007
 *********************************************************************/

#include "carsmanager.hpp"
#include "terraingenerator.hpp"
#include "weaponmanager.hpp"
#include "towermanager.hpp"
#include "collisionmanager.hpp"
#include "supramanager.hpp"

#include "solardiales.hpp"

#include <algorithm>
#include <iostream>
int CarsManager::instances = 0;

// ------------------------------------------------
// class "Car"
// ------------------------------------------------

// il revient a l'utilisateur d'updater les voitures
// qu'il a instancié.
void Car::Update(float delta_time)
{
	Car copy;
	this->PartialCopyInto(&copy);	 // on copie l'etat actuel au cas ou le mouvement est refuse par une collision

	// direction :
	angle += GetWheelFactor() * delta_time * speed;		 // radians/s
	direction = Rotate(Vector2f(0.f, -1.f), angle);
	// position :
	posf += direction * speed * SPEED_MULTIPLIER * delta_time;	// calcul necessairement floating point
	// cast en pixels :
	rect.pos.x = (int)posf.x;
	rect.pos.y = (int)posf.y;

	// calcul du ORect:
	ORect orc = GetORect();

	// si collision avec le joueur : petite explusion et prise de dégats :
	bool isplayer = &*game_vars::g_user == this;
	ORect playrRc = game_vars::g_user->GetORect();
	if (!isplayer && playrRc.IntersectsORc(orc) && game_vars::g_user->health > 0)
	{
		// calculer le vecteur de direction d'explusion
		Vector2f expuls = rect.GetCenter() - playrRc.rect.GetCenter();
		expuls *= 0.2f;

		// explusion :
		posf += expuls;

		// degats :
		InflictDamage(20);
		// dégats sur le joueur :
		game_vars::g_user->InflictDamage(20);

		// donner des points au joueur s'il est mort :
		if (health <= 0)
			game_vars::g_user->SignalKills(1);

		// et on refait le rect :
		rect.pos.x = (int)posf.x;
		rect.pos.y = (int)posf.y;
		orc = GetORect();
	}


	Vector2f dirbackup = direction;
	float anglebackup = angle;

	// si collision, rollback dans l'état précedent la tentative de mouvement :
	if (ChiefManager::GetCollisionsAndProbing()->IntersectORect(orc))
	{
		// problème : si on a pas le droit de tourner en collision, on est coincé pour toujours.
		// workaround : si l'user veut tourner, testons si les positions decalées d'un pixel
		//					 dans 4 directions sont libres.

		if (copy.angle != anglebackup)  // si on veut tourner
		{
			int ddx[4] = {0, 1, -1, -1};
			int ddy[4] = {-1, 1, 1, -1};
			int dx[4] = {0, 1, 0, -1};
			int dy[4] = {-1, 0, 1, 0};
			bool free = false;
			for (int i = 0; i < 4; ++i)  // tester 4 déplacements
			{
				orc.rect.pos.x += ddx[i];
				orc.rect.pos.y += ddy[i];

				if (!ChiefManager::GetCollisionsAndProbing()->IntersectORect(orc))  // liiiibre !
				{
					posf.x += dx[i];
					posf.y += dy[i];
					rect.pos.x = (int)posf.x;
					rect.pos.y = (int)posf.y;
					free = true;
					break;
				}
			}
			if (!free)
			{ // bon ben solution de secours:
				copy.PartialCopyInto(this);
				angle = anglebackup;
				direction = dirbackup;
			}
		}
		else
			copy.PartialCopyInto(this);  // blocage
	}
}

void Car::PartialCopyInto(Car* dest)
{
	dest->posf = this->posf;
	dest->angle = this->angle;
	dest->direction = this->direction;
	dest->rect = this->rect;
}

// destruction
Car::~Car()
{
	// lorsque une voiture disparait (explose, passe la fin du niveau, est enlevee par les extraterrestres...)
	// on desactive les trackers qui nous avaient pris pour cible sinon les pointeurs qu'ils stockent
	// vont devenir obsoletes et des problemes apparaîtrons.
	TwrListIt it;
	for (it = trackers.begin(); it != trackers.end(); ++it)
	{
		(*it)->currentTarget = 0;
	}
}

// ajoute une tourelle dans la liste des trackers actifs réferencés.
void Car::RegisterTracker(TowerInstance* ti)
{
	trackers.push_back(ti);
}

// retire une tourelle de la liste des trackers
void Car::UnregisterTracker(TowerInstance* ti)
{
	TwrListIt iter = std::find(trackers.begin(), trackers.end(), ti);
	if (iter != trackers.end())  // j'ai pas vu le comportement de l.erase(l.end()) dans le standard
		trackers.erase(iter);
}


// ------------------------------------------------
// class "CarsManager"
// ------------------------------------------------

void CarsManager::DrawCars(void)
{
	// benchmark avec 4000 ennemis par minutes :
	//	  113 FPS avec cette fonction, 115 sans.
	// aucun interet donc de faire une structure
	// de partitionnement d'espace.
	// probablement parce que de toute facon le manager d'hostiles
	// boucle sur tous ses pions, rendant ceci negligeable devant l'IA.

	// dessin des epaves en premier car les autres voitures doivent "rouler dessus".
	for (int i = 0; i < wrecks.size(); ++i)
	{
		// positionner les sprites :
		sprites.Move(wreckId, wrecks[i].x, wrecks[i].y);
		// et dessiner :
		sprites.Draw(wreckId);
	}
	
	// dessin des voitures
	for (CarIT it = cars.begin(); it != cars.end(); ++it)
	{
		if (it->car.health > 0)
		{
			sprites.Rotate(it->spriteId, -ToDegree(it->car.angle));
			sprites.Move(it->spriteId, it->car.rect.pos.x, it->car.rect.pos.y);
			sprites.Draw(it->spriteId);
			// sa position dans le cadre du niveau:
			int spid = 0;
			if (it->car.GetIAClassID() == game_vars::g_simpleIAClassID)
			{
				spid = pos_view.redPixelId;
			}
			else if (it->car.GetIAClassID() == game_vars::g_strongIAClassID)
			{
				spid = pos_view.yellowPixelId;
			}
			if (spid > 0)
			{
				// calculer la position du pixel dans le cadre:
				int x = it->car.GetRawPosf().x / pos_view.level_size.x * 80 + 200;
				int y = it->car.GetRawPosf().y / pos_view.level_size.y * 40 + 8;
				pos_view.pnoscroll_spm->Draw(spid, x, y);
			}
		}
	}

	// et enfin dessin des explosions en cours :
	double now = time;
	for (int i = 0; i < bombs.size(); ++i)
	{
		sprites.Draw(explodeId, bombs[i].pos.x, bombs[i].pos.y, now - bombs[i].stamp);
		if (now > bombs[i].stamp + 0.7)
			bombs.pop(i);	 //on l'enleve de la liste quand l'animation est entierement jouee
	}
}

void CarsManager::Explode(CarAcsor& acsor, bool dontInvalidate /*= false*/)
{
	// on push une explosion et un wreckage dans les listes :
	Vector2i location = acsor.it->car.rect.pos;
	bombs.push_back(PosAndTime(location, time));
	wrecks.push_back(location);
	ChiefManager::GetSoundManager()->ReadSound("explosion", 1, true, location);
	
	if (!dontInvalidate) // on supprime la voiture si le client de la fonction ne veut pas la garder
		Pop(acsor);
}

void CarsManager::SetExplosionSprite(char const* fname)
{
	// chargement du sprite d'explosion :
	explodeId = sprites.LoadSprite(fname);
	sprites.SetSpeed(explodeId, 8);	 // 8 FPS
}

void CarsManager::SetWreckageSprite(char const* fname)
{
	// meme chose pour le wreckage :
	wreckId = sprites.LoadSprite(fname);
}

// ---------------------------------
// si l'utilisateur ne se sert QUE d'un CarsManager dans tout le programme
// (et donc d'aucun autre sprite manager) il peut vouloir rafraichir
// la surface SDL, donc il faut permettre un accès a Update.
void CarsManager::Update(double global_time, UPDATE_PARAM_en p /*= UP_TIME_ONLY*/)
{
	time = global_time;
	sprites.UpdateWithClientTime(global_time, p);  // oui, car 'Update - time only' ne sert a rien du tout pour les sprites rotatés.
}

// ---------------------------------
// ajouter une voiture
CarsManager::CarAcsor CarsManager::Push(char const* sprite)
{
	cars.push_front(ManagedCar());  // on pousse en tete pour pouvoir acceder a l'iterateur en temps constant

	// cherchons ce nom de fichier dans notre map :
	MapIT mit = fn2idMap.find(sprite);
	// s'il n'existe pas
	if (mit == fn2idMap.end())
	{
		// les sprites de voitures sont de type "rotated"
		int sid = sprites.LoadSprite(sprite, SP_ROTATED);	// on creer un nouveau sprite
		// insertion dans la map :
		mit = fn2idMap.insert(MapVType(sprite, sid)).first;
	}
	// renseignement des champs de la nouvelle voiture :
	cars.front().spriteId = mit->second;
	// renseignement du rect :
	cars.front().car.rect.w = sprites.GetW(cars.front().spriteId);
	cars.front().car.rect.h = sprites.GetH(cars.front().spriteId);
	// on retourne un accesseur :
	return CarAcsor(cars.begin());  // construit avec l'iterateur, qui ne sera jamais invalidé. (voir doc SGI)
}

// ---------------------------------
// suppression d'une voiture dans la liste:
void CarsManager::Pop(CarAcsor& acsor)
{
	if (acsor.IsValid())
	{
		cars.erase(acsor.it);  // temps constant, n'invalide pas les autres iterateurs sur la liste 'cars'.
		acsor.InvalidateClass();
	}
}
