/**********************************************************************
 * File	: highscore.cpp
 * Author: Lightness1024
 *			  mailto:lightness1024@free.fr
 * Date	: 02/05/2008
 *********************************************************************/

#include "highscore.hpp"
#include <string.h>

// ------------------------------------------------
// fonctions d'écriture de type de base
// ------------------------------------------------

static void WriteInt(FILE* stream, int val)
{
	fwrite(&val, sizeof(int), 1, stream);
}

static int ReadInt(FILE* stream)
{
	int val;
	fread(&val, sizeof(int), 1, stream);
	return val;
}

// static void WriteString(FILE* stream, const char* str)
// {
// 	int strsz = strlen(str);
// 	WriteInt(stream, strsz);
// 	fwrite(str, sizeof(char), strsz, stream);
// }

static void WriteString(FILE* stream, std::string str)
{
	WriteInt(stream, str.length());
	fwrite(str.c_str(), sizeof(char), str.length(), stream);
}

static std::string ReadString(FILE* stream)
{
	int sz = ReadInt(stream);
	char* tmp = new char[sz + 1];
	fread(tmp, sizeof(char), sz, stream);
	tmp[sz] = 0;
	std::string str(tmp);
	delete [] tmp;
	return str;
}

// ------------------------------------------------
// implémentation de la structure ScoreRecord
// ------------------------------------------------

// méthode de chargement d'un ScoreRecord à partir d'un fichier
void ScoreRecord::ReadFromFile(FILE* is)
{
	int version = ReadInt(is);
	if (version == 1)
	{
		playerName = ReadString(is);
		score = ReadInt(is);
	}
}

// méthode de sérialisation d'un ScoreRecord vers un fichier
void ScoreRecord::WriteToFile(FILE* os) const
{
	// version :
	WriteInt(os, 1);
	// membres:
	WriteString(os, playerName);
	WriteInt(os, score);
}

// ------------------------------------------------
// implémentation de la classe manager de scores
// ------------------------------------------------

// fonction de chargement d'un fichier de highscore
void HighscoreManager::UseHighScoreFile(const char* filename)
{
	// désérialisation (lecture) :
	
	fileUpToDate = false;
	workingFilename = filename;

	FILE* is = fopen(filename, "rb");
	if (is != 0)  // si ok
	{
		int version = ReadInt(is);
		if (version == 1 && !feof(is))
		{
			int numRecords = ReadInt(is);
			// boucler sur les enregistrements de highscores
			for (int i = 0; i < numRecords && !feof(is); ++i)
			{
				ScoreRecord rec;
				rec.ReadFromFile(is);
				records.insert(rec);
			}
		}
		fclose(is);
	}
}

void HighscoreManager::WriteOnFile(void)
{
	// écriture des informations contenues dans la classe sur le fichier de sortie
	
	if (!fileUpToDate)
	{
		FILE* os = fopen(workingFilename.c_str(), "wb");
		if (os != 0)  // si ouverture ok
		{
			// écriture de la version:
			WriteInt(os, 1);
			// écriture du nombre de records :
			WriteInt(os, GetScoreNum());
			// puis écriture de tous les enregistrements:
			set< ScoreRecord >::iterator it;
			for (it = records.begin(); it != records.end(); ++it)
			{
				it->WriteToFile(os);
			}
		}
		fclose(os);
		fileUpToDate = true;
	}
}


ScoreRecord HighscoreManager::GetScoreRec(int position) const
{
	set< ScoreRecord >::const_iterator it;
	int i;
	for (it = records.begin(), i = 0 ;
		  it != records.end() && i != position;
		  ++it, ++i)
	{
	}
	return *it;
}

size_t HighscoreManager::GetScoreNum(void) const
{
	return records.size();
}

void HighscoreManager::AddScore(ScoreRecord record)
{
	records.insert(record);
	fileUpToDate = false;
}

// destructeur
HighscoreManager::~HighscoreManager()
{
	WriteOnFile();
}

/*
// main de test
#include <iostream>
using namespace std;
// TMP

int main(void)
{
	HighscoreManager mg;
	mg.AddScore(ScoreRecord("badguy", 450));
	mg.AddScore(ScoreRecord("morph", 601));
	mg.AddScore(ScoreRecord("badguy", 4500));
	mg.AddScore(ScoreRecord("morph", 6010));
	mg.AddScore(ScoreRecord("morph", 10));
	mg.AddScore(ScoreRecord("morph", 6));
	
	cout << "getscnum : " << mg.GetScoreNum() << endl;
	for (int i = 0; i < mg.GetScoreNum(); ++i)
		cout << "get sc rec " << i << ": " << mg.GetScoreRec(i).score << " pn: " << mg.GetScoreRec(i).playerName << endl;
	
	return 0;
}
*/
