/**********************************************************************
 * File	: highscore.hpp
 * Author: Lightness1024
 *			  mailto:lightness1024@free.fr
 * Date	: 29/04/2008
 *********************************************************************/

#ifndef HIGHSCORE_HPP_INC_19_03_2009___250786323_
#define HIGHSCORE_HPP_INC_19_03_2009___250786323_ 1

#include <string>
using std::string;
#include <set>
using std::set;

#include <stdio.h>	// de temps en temps marre des fstream, vive un bon vieux fwrite/fread

// ---------------------------
// structure stockant un score
// ---------------------------
struct ScoreRecord
{
	ScoreRecord(const char* playnme = "", int scr = 0)
			: playerName(playnme),
			  score(scr)
	{ }

	string playerName;
	int score;

	bool operator < (const ScoreRecord& op) const
	{
		if (score == op.score)
			return playerName > op.playerName;
		else
			return score > op.score;
	}
		
	void ReadFromFile(FILE* is);
	void WriteToFile(FILE* os) const;
};

// ---------------------------
// classe manager de scores
// ---------------------------
class HighscoreManager
{
	int maxScoreNum;
	string workingFilename;
	set< ScoreRecord > records;
	bool fileUpToDate;

public:

	void UseHighScoreFile(const char* filename);          // changer le fichier utiliser/lire à partir de ce fichier
	void WriteOnFile(void);								  // écrire sur le fichier (appelé auto dans le destruct)

	ScoreRecord GetScoreRec(int position) const;		  // recupère une ligne de highscore
		
	size_t GetScoreNum(void) const;						  // recupère le nombre de highscores
	void AddScore(ScoreRecord record);					  // ajoute un highscore au tableau
		
	void AddScore(const char* playerName, int score)	  // binding
	{
		AddScore(ScoreRecord(playerName, score));
	}

	// constructeur :
	HighscoreManager(const char* filename = "scores.bin", int a_maxScoreNum = 5)
			: maxScoreNum(a_maxScoreNum),
			  fileUpToDate(false)
	{
		UseHighScoreFile(filename);
	}
		
	// destructeur :
	~HighscoreManager();
};

#endif //HIGHSCORE_HPP_INC_19_03_2009___250786323_
