/**********************************************************************
 * File	: hud.hpp
 * Author: Vivien - a.k.a. Lightness1024!
 *			  mailto:lightness1024@free.fr
 * Date	: 15/07/2007
 *********************************************************************/

#ifndef HUD_HPP_INC_19_03_2009__8387165_
#define HUD_HPP_INC_19_03_2009__8387165_ 1

#include "spritemanager.hpp"
#include "noncopyable.hpp"

#include <vector>

using std::vector;

enum MENU_CURSOR_MOVE_en
{
	M_CUR_LEFT,
	M_CUR_RIGHT,
	M_CUR_STAY
};

//-------------------------------------------------------
// Menu
//-------------------------------------------------------
class Menu
{
	SpriteManager spm;  // maintenir un sprite manager

	struct MenuElement
	{
		int sprId;
		int posy;
		int height;
	};
	// elements du menu de droite :
	vector< MenuElement > r_elements;	// right elements
	// partie de gauche :
	vector< MenuElement > l_elements;	// left elements

	// note : en pratique dans le jeux, les elements de droite sont les armes
	//			 et les elements de gauche sont les tourelles.
	//			 je prevois la possibilite d'utiliser cette classe de menu
	//			 en specifiant des elements uniquement d'un seul cote
	//			 si il y a besoin de faire un menu simple. par exemple pour lancer une partie.

	int arrowId;
	int cursorPos;
	int rx, ry, lx, ly;
	int spacing;
	int headleft, headright;
	
	void PlaceArrow(void);

public:

	int SetHeadLeft(int x, int y, const char* sprite);
	int SetHeadRight(int x, int y, const char* sprite);

	int PushLeft(const char* sprite);
	int PushRight(const char* sprite);

	int GetLeftSize(void);
	int GetRightSize(void);
	
	Vector2i GetElemPos(int id);
	Vector2i GetElemSize(int id);

	void SetArrow(const char* fn_arrow);

	void SignalCursorMove(MENU_CURSOR_MOVE_en direction);
	int GetCursorPosition(void);
	void SetCursorPosition(int pos);
	
	enum EDrawType
	{
		WITH_FLIP,
		NO_FLIP
	};
	void Draw(EDrawType dt = WITH_FLIP);

	void SetRightStartPos(int x, int y);
	void SetLeftStartPos(int x, int y);
	void SetVerticalSpacing(int s);

	Menu(void)
			: arrowId(-1),
			  cursorPos(0),
			  rx(400),
			  ry(110),
			  lx(175),
			  ly(110),
			  spacing(10),
			  headleft(-1),
			  headright(-1)
	{
		spm.SetScrollSensible(false);
	}
};


//-------------------------------------------------------
// Numbers
//-------------------------------------------------------
class Numbers : boost::noncopyable
{
	SpriteManager spm;

	int w;

public:

	inline void LoadNumbers(const char* file);

	void PrintNum(int x, int y, unsigned int value);
};

void Numbers::LoadNumbers(const char* file)
{
	spm.SetScrollSensible(false);
	spm.LoadSprite(file);
	int kerning = spm.GetW(0) / 6;
	w = spm.GetW(0) - kerning;	 // kerning negatif pour resserer l'affichage des chiffres entre eux
	spm.SetSpeed(0, 1.0001f);
}

//-------------------------------------------------------
// Hud
//-------------------------------------------------------
class Hud
{
	SpriteManager sprMngr;	      // mister sprite

	vector< smart_ptr< Numbers > > numbers;   // managers d'affichage de nombres (peut en avoir plusieur car plusieurs polices)

	struct Element
	{
		int x, y;               // position a l'ecran en pixel
		int current;            // indice de l'element courant
		vector< int > spIds;    // tableau des elements alternatifs. (un element de hud peut alterner plusieurs sprites)

		int numberFontID;       // est-ce un nombre ? (score, argent...) si oui stocker l'id du manager de font
		unsigned int value;

		bool visible;

		Element(void)
				: current(0),
				  numberFontID(-1),
				  value(0),
				  visible(true) { }
	};

	vector< Element > elements;	// collection d'elements sur le hud.

	vector< vector< int > > progressBars;

	void DrawSpecialProgressBar(Element const & e);

public:

	// retourne un id vers un element de base :
	int PushNewHudElement(int x, int y, const char* fname, int numberFontID = -1, bool animated = false);
	// retourne un id vers un element alterne d'un element de base :
	int PushAlternateHudElement(int baseElement, const char* fname);

	void SetElementVisible(int elemId, bool visible);
	void UpdateNumberValue(int elemId, unsigned int value);
	void UpdateAlternateElement(int baseElement, int alternateElement);

	int PushNewNumbersFontSprite(const char* file);

	enum EPBarType
	{
		PB_STANDARD,
		PB_WITH_ACTIVATED_PARTS
	};
	int PushProgressBar(int x, int y, const char* bar, const char* criticalBar, EPBarType etype = PB_STANDARD);
	void UpdateProgressBar(int barId, int progress);

	int PushAnimatedElement(int x, int y, const char* fname)
	{
		return PushNewHudElement(x, y, fname, -1, true);
	}

	void Draw(void);

	Hud(void)
	{
		elements.reserve(10);
		sprMngr.SetScrollSensible(false);
	}
};

#endif //HUD_HPP_INC_19_03_2009__8387165_
