/**********************************************************************
 * File  : proximity_manager.hpp
 * Author: Lightness1024
 *         mailto:lightness1024@free.fr
 * Date  : 23/11/2008
 *********************************************************************/

#ifndef PROXIMITY_MANAGER_HPP_INC_19_03_2009___188271289_
#define PROXIMITY_MANAGER_HPP_INC_19_03_2009___188271289_ 1

#include "array.hpp"
#include "carsmanager.hpp"
#include "soundmanager.hpp"   // pour avoir MAX_[..]DISTANCE

enum EKindOfDistance
{
	PM_NORMAL_DIST,
	PM_SQUARED_DIST
};

class ProximityManager
{
public:

	typedef boost::array< CarsManager::CarAcsor, 5 > CarsHardArray;

	ProximityManager(void)
			: head(0),
			  radius(MAX_LISTENING_DISTANCE)
	{ }

	void ConditionalAdd(CarsManager::CarAcsor const & car, float distance, enum EKindOfDistance kind = PM_NORMAL_DIST)
	{
		float against = radius;
		if (kind == PM_SQUARED_DIST)
			against = against * against;

		if (distance < against)
		{
			Add(car);
		}
	}

	void Add(CarsManager::CarAcsor const & car)
	{
		if (head < proximity_list.size())
		{
			proximity_list[head] = car;
			++head;
		}
	}

	void Reset(void)
	{
		for (size_t i = 0; i < head; ++i)
		{
			proximity_list[i] = CarsManager::CarAcsor();
		}
		head = 0;
	}
	
	CarsHardArray GetCarsInProximity(void)
	{
		return proximity_list;
	}

	void SetRadius(float rad)
	{
		radius = rad;
	}

	float GetRadius(void)
	{
		return radius;
	}

	CarsManager::CarAcsor& GetNearestFrom(Vector2i const & pos)
	{
		size_t index = 0;
		float distSq = 1e36f;

		for (size_t i = 0; i < head; ++i)
		{
			float curDist = DistTwoPtsSq(proximity_list[i]->rect.GetCenter(), pos);
			if (curDist < distSq)
			{
				index = i;
				distSq = curDist;
			}
		}
		return proximity_list[index];
	}



	CarsManager::CarAcsor& operator [] (int index)
	{
		return proximity_list.operator[](index);
	}

private:

	CarsHardArray proximity_list;
	size_t head;
	float radius;
};

#endif //PROXIMITY_MANAGER_HPP_INC_19_03_2009___188271289_
