/**********************************************************************
 * File	: towermanager.hpp
 * Author: Lightness1024!
 *			  mailto:lightness1024@free.fr
 * Date	: 31/07/2007
 *********************************************************************/

#ifndef TOWERMANAGER_HPP_INC_19_03_2009__126269260_
#define TOWERMANAGER_HPP_INC_19_03_2009__126269260_ 1

#include "spritemanager.hpp"
#include "hostilesmanager.hpp"
#include "weaponmanager.hpp"

#include "freelist.hpp"	  // yaiiiiy ca y'est j'ai programmé une freelist, ca poutre.
using tools::freelist;

#include "geometry.hpp"
#include "smart_ptr.hpp"
#include "SecureArray.hpp"
#include "gunner.hpp"

#include <map>
using std::map;

// ----------------------------------------------------
// toutes les informations definissants un type de tourelle
// ----------------------------------------------------
struct Tower
{
	int spriteId;
	int weaponId;
	int price;

	Tower(void)
	{}
	Tower(int sid, int wid, int pr)
			: spriteId(sid),
			  weaponId(wid),
			  price(pr)
	{}
};

// ----------------------------------------------------
// instance de tourelle, une tourelle est aussi un tireur.
// ----------------------------------------------------
struct TowerInstance : public Gunner
{
	int towerKind;			 // type de tourelle
	Gunner* builder;		 // pointeur vers le "constructeur" (le joueur) qui beneficiera des frags de cette tourelle.
	Car* currentTarget;	     // pour l'intelligence artificielle de visee

	HostilesManager::FireRange firingRange;	// zone de tir autour de la tourelle.
	// concretement contient une collection
	// de cellules de la gille d'hostiles.

	TowerInstance(void)
	{
		istower = true;  // on initialise pas un membre de classe parente dans la liste d'init
	}
	TowerInstance(int tk, Gunner* gnr)
			: towerKind(tk),
			  builder(gnr),
			  currentTarget(0)
	{
		istower = true;
	}
};

// ----------------------------------------------------
// Tower Space Partitionning Cell
// ----------------------------------------------------
struct TSPCell
{
	// pour les tourelles, la freelist est parfaite vu qu'on ne detruit jamais une tourelle construite
	freelist< TowerInstance > towers;
	typedef freelist< TowerInstance >::iterator TwIt;

	TSPCell(void)
			: towers(10) { } // on règle l'heuristique de preallocation de la freelist a 10 elements.
};

// ----------------------------------------------------
// gestionnaire de tourelles
// ----------------------------------------------------
class TowerManager
{
	SpriteManager sprMgr;

	// base de donnée de types de tourelles
	SecureArray< Tower > towerKinds;

	int maxw, maxh;  // dimensions de la tourelle la plus grosse de la grille.
	const int log2cellsize;

	// smart pointer vers une cellule:
	typedef smart_ptr< TSPCell > TSPCellSptr;

	// foncteur de comparaison de vecteurs
	struct CmpVctorFctor
	{
		bool operator () (const Vector2i& v1, const Vector2i& v2) const
		{
			if (v1.y < v2.y)
				return true;
			else if (v1.y == v2.y)
				return v1.x < v2.x;
			else
				return false;
		}
	};

	// grille de partitionnement d'espace et par la meme occasion stockage des tourelles :
	map< Vector2i, TSPCellSptr, CmpVctorFctor > cells;
	// typedefs : iterator et value_type.
	typedef map< Vector2i, TSPCellSptr, CmpVctorFctor >::iterator ClIt;
	typedef map< Vector2i, TSPCellSptr, CmpVctorFctor >::value_type ClValT;

	static int instances;

	void UpdateIAOnTower(TSPCell::TwIt& twrit);

	void UpdateTargetReachability(TSPCell::TwIt& tower);
	void FindTarget(TSPCell::TwIt& tower);
	void RefreshFireDir(TSPCell::TwIt& tower);

public:

	// créer un type de tourelle :
	int CreateNewKindOfTower(const char* sprite, int weaponId, int price);

	// renseigner du prix :
	int GetPrice(int towerId)
	{
		return towerKinds[towerId].price;
	}

	// construire une instance de tourelle :
	bool BuildNewTower(int x, int y, int kindOfTower, Gunner* builder);

	// déterminer l'intersection entre un orect et une tourelle :
	bool TowerIntersectORect(const ORect& orc, bool bounding = false);

	// déterminer l'intersection entre un rect et une tourelle (plus rapide) :
	bool TowerIntersectRect(const Rect& rc)
	{
		return TowerIntersectORect(rc, true);
	}

	// intersection entre rayon de probing et une tourelle :
	bool TowerIntersectProbeRay(float* t, const Vector2f& start, const Vector2f& direction);

	// affichage, IA
	void Update(double gtime);

	struct TooManyInstances {};

	// construction de la classe twr manager :
	TowerManager(void)
			: maxw(0),
			  maxh(0),
			  log2cellsize(9)	 // grille de 512 pixels
	{
		if (++instances != 1)
			throw TooManyInstances();
		towerKinds.ReserveExtra(3);
	}
	~TowerManager()
	{
		--instances;
	}
};

#endif //TOWERMANAGER_HPP_INC_19_03_2009__126269260_
