/**********************************************************************
 * File  : sharedsurface.cpp
 * Author: Lightness1024
 *         mailto:lightness1024@free.fr
 * Date  : 07/03/2009
 *********************************************************************/

#include "sharedsurface.hpp"
#include <SDL/SDL.h>

namespace render2d
{
	//---------------------------------------------
	// class: SDLLife
	//---------------------------------------------
	bool SDLLife::initDone = false;  // static
	bool SDLLife::quitDone = false;  // static
	EDestroyPolicy SDLLife::firstPolicy = DP_NO_POLICY;  // static
	int SDLLife::instances = 0;  // static

	SDLLife::SDLLife(EDestroyPolicy policy /*= DP_QUIT_SDL*/)
	{
		if (firstPolicy == DP_NO_POLICY)
		{
			firstPolicy = policy;
		}
		ManuallyInitSDL();
		++instances;
	}

	// static
	void SDLLife::ManuallyInitSDL()
	{
		if (quitDone)
		{
			throw Cant_Use_SDL_After_It_Has_Been_Quit();
		}
		if (!initDone)
		{
			SDL_Init(SDL_INIT_VIDEO);
			initDone = true;
		}
	}

	// static
	void SDLLife::ManuallyQuitSDL()
	{
		if (instances != 0)
			throw Cant_Quit_SDL_While_Lives_Exist();
		
		if (!quitDone)
		{
			SDL_Quit();
			quitDone = true;
		}
	}

	SDLLife::~SDLLife()
	{
		--instances;
		if (instances == 0)
		{
			if (firstPolicy == DP_QUIT_SDL)
			{
				ManuallyQuitSDL();
			}
			firstPolicy = DP_NO_POLICY;
		}
	}

	//---------------------------------------------
	// class: Sharedsurface
	//---------------------------------------------
	SDL_Surface* SharedSurface::surf = 0; // static
	int SharedSurface::x_ = 0;
	int SharedSurface::y_ = 0;

	SharedSurface::SharedSurface(int x, int y, EDestroyPolicy p)
			: SDLLife(p)
	{
		if (!surf)
		{
			surf = SDL_SetVideoMode(x, y, 0, SDL_HWSURFACE | SDL_DOUBLEBUF);
			if (!surf)
				throw Cant_Create_Window();
			x_ = x;
			y_ = y;
		}
	}
	
	// le constructeur par defaut construit une surface faible referencant la vraie existante
	SharedSurface::SharedSurface()
			: SDLLife(DP_WEAK)
	{
		if (!Exists())
			throw No_Reference_Surface();  // erreur d'utilisation

	}
	
	SharedSurface::~SharedSurface()
	{
		if (GetInstances() == 1)  // dernière survivante
		{
			surf = 0;
			x_ = 0;
			y_ = 0;
		}
	}
}
