/**********************************************************************
 * File	: solardiales.cpp
 * Author: Vivien - a.k.a. Lightness1024!
 *			  mailto:lightness1024@free.fr
 * Date	: 09/07/2007
 *********************************************************************/


#include "randclass.hpp"
#include "myutils.hpp"

#include "spritemanager.hpp"

#include "hostilesmanager.hpp"
#include "geometry.hpp"
#include "weaponmanager.hpp"
#include "towermanager.hpp"
#include "collisionmanager.hpp"
#include "proximity_manager.hpp"
#include "supramanager.hpp"
#include "soundmanager.hpp"
#include "fpscomputer.hpp"
#include "highscore.hpp"
#include "sharedsurface.hpp"
#include "makethread.hpp"
#include "performanceprofiler.hpp"

#include "game_menus.hpp"
#include "screen_inertia.hpp"
#include "sdl_events.hpp"

#include "solardiales.hpp"

#include <iostream>
#include <fstream>
#include <sstream>
#include <string.h>
#include <time.h>


#define DEFAULT_RES_X 1024
#define DEFAULT_RES_Y 768
#define DEFAULT_MUSIC_VOLUME 50
#define CONF_FILE_NAME "conf.ini"


// *** quelques globales bien degeux ***

namespace game_vars
{
	int gameover;
	bool g_score_is_acceptable = true;
	Vector2f g_scrPos(0, 0);
	CarsManager::CarAcsor g_user;
	bool g_cheat = false;
	int g_currentWeapon = evil_value;
	bool g_keyleft = false;
	bool g_keyright = false;
	int g_minigun_id = evil_value;
	int g_heavymg_id = evil_value;
	int g_rocketl_id = evil_value;
	int g_superfg_id = evil_value;
	int g_qsuperfg_id = evil_value;
	bool g_carControlDevice = false;
	size_t g_playerIAClassID = evil_value;
	size_t g_simpleIAClassID = evil_value;
	size_t g_strongIAClassID = evil_value;

	namespace shop
	{
		bool g_menuActivated;
		Menu* g_pmenu;
		int g_ccd_id;  // id des upgrades
		int g_shield_id;
		NumToIDs g_key_to_id;
	}

	void Reset()
	{
		gameover = 0;
		g_score_is_acceptable = true;
		g_scrPos = Vector2f::Zero();
		g_user = CarsManager::CarAcsor();
		g_currentWeapon = evil_value;
		g_keyleft = false;
		g_keyright = false;
		g_minigun_id = evil_value;
		g_heavymg_id = evil_value;
		g_superfg_id = evil_value;
		g_qsuperfg_id = evil_value;
		g_rocketl_id = evil_value;
		g_carControlDevice = false;
		g_playerIAClassID = evil_value;
		g_simpleIAClassID = evil_value;
		g_strongIAClassID = evil_value;

		shop::g_menuActivated = false;
		shop::g_pmenu = 0;
		shop::g_ccd_id = evil_value;
		shop::g_shield_id = evil_value;
		shop::g_key_to_id.clear();
	}
}

void SetEngineSound(ESESSetOnOff setting)
{
	// on arrete tous les sons de moteur :
	ChiefManager::GetSoundManager()->StopSound("engine-fast");
	ChiefManager::GetSoundManager()->StopSound("engine-normal");
	ChiefManager::GetSoundManager()->StopSound("engine-idle");

	if (setting == SES_ON && game_vars::g_user->health > 0)
	{
		// et réglage du son correct :
		if (game_vars::g_user->speed >= GetFastSpeed() || game_vars::g_user->speed <= -GetFastSpeed())
		{
			// go fast
			ChiefManager::GetSoundManager()->ReadSound("engine-fast", 0);
		}
		else if (game_vars::g_user->speed >= GetNormalSpeed() || game_vars::g_user->speed <= -GetNormalSpeed())
		{
			// go normal
			ChiefManager::GetSoundManager()->ReadSound("engine-normal", 0);
		}
		else if (game_vars::g_user->speed == 0.0f)
		{
			// go idle
			ChiefManager::GetSoundManager()->ReadSound("engine-idle", 0);
		}
	}
}

void SetPassingCarSound(void)
{
	static bool s_lastNearbyWasValid = false;
	static int s_channel = -1;

	// récupère le plus proche des ennemis proche :=)
	CarsManager::CarAcsor& nearby = ChiefManager::GetProximityManager()->GetNearestFrom(game_vars::g_user->rect.GetCenter());

	if (s_lastNearbyWasValid && nearby.IsValid())   // cas de figure : proche[0] est valide ainsi que l'ancien, donc le son est deja démarré
	{  // il faut updater la position du son lu
		ChiefManager::GetSoundManager()->UpdateSoundPosition("engine-muffled", nearby->rect.GetCenter(), s_channel);
	}
	else if (!s_lastNearbyWasValid && nearby.IsValid())  // cas de figure : il n'y avait plus de son a l'image d'avant et le nouveau proche[0] est valide
	{  // démarrer le son
		s_channel = ChiefManager::GetSoundManager()->ReadSound("engine-muffled", -1, true, nearby->rect.GetCenter(), true);
		s_lastNearbyWasValid = true;
	}
	else if (!nearby.IsValid() && s_lastNearbyWasValid) // cas de figure : le nouveau n'est pas valide
	{  // plus rien a lire
		ChiefManager::GetSoundManager()->StopSound("engine-muffled");
		s_lastNearbyWasValid = false;
	}
}

int GetIndexOfPlayer(std::string const& name)
{
	if (name == "badguy")
		return 0;
	if (name == "morph")
		return 1;
	else
		return -1;
}

char const * GetPlayerFromIndex(int index)
{
	if (index == 0)
		return "badguy";
	if (index == 1)
		return "morph";
	else
		return "";
}

std::string MakeBmpFN(char const* name)
{
	return std::string("resources/hud/players/") + name + ".bmp";
}


void ReadConfFile(int* resxOut, int* resyOut, int* musicVolume)
{
	std::ifstream s(CONF_FILE_NAME, std::ifstream::in);
	if (s.good())
	{
		s >> *resxOut;
		s >> *resyOut;
		s >> *musicVolume;
		*musicVolume = myutils::clamp(*musicVolume, 0, 100);
	}
	else
	{
		*resxOut = DEFAULT_RES_X;
		*resyOut = DEFAULT_RES_Y;
		*musicVolume = DEFAULT_MUSIC_VOLUME;
	}
	s.close();
}

int StartGame(GameInfos infos)
{
	render2d::SharedSurface gameSurface(infos.resx, infos.resy, render2d::DP_WEAK);
	SpriteManager sprmngr("extremecarnage - playing");

	Screen screen(SpriteManager::GetResX(), SpriteManager::GetResY(), 0, 0);
	tools::Random rgen;

	rgen.SetSeed(infos.param.seed);

	// instanciation des managers de jeu :
	TerrainManager terrain;
	TowerManager towermanager;
	CarsManager carmngr;
	HostilesManager enemies;
	WeaponManager arms;
	CollisionsAndProbing collider;
	SoundManager sounds(SoundManager::CP_DONT_CLOSE);
	ProximityManager proximity;
	HighscoreManager highScoresMgr;
	ia::Manager<3> iaManager;

	ChiefManager::SetManagersPointers(&terrain,
									  &carmngr,
									  &enemies,
									  &arms,
									  &towermanager,
									  &collider,
									  &sounds,
									  &proximity,
									  &highScoresMgr,
									  &iaManager);


//=============================== init jeu =============================

	Hud hud;
	Menu shop_menu;
	game_vars::shop::g_pmenu = &shop_menu;
	sprmngr.SetScrollSensible(false);

// --- creation du hud : ---

	hud.PushNewNumbersFontSprite("resources/hud/numbers.bmp");  // id 0

	// carnage points en haut a gauche
	hud.PushNewHudElement(10, 10, "resources/hud/carnage_pts_hud.bmp");
	// nombre de carnage points :
	int scoreHudId = hud.PushNewHudElement(10, 52, NULL, 0);

	// on push chaque weapon :
	int bid = hud.PushNewHudElement(10, 110, "resources/hud/mg_hud.bmp");
	hud.PushAlternateHudElement(bid, "resources/hud/hmg_hud.bmp");
	hud.PushAlternateHudElement(bid, "resources/hud/rl_hud.bmp");
	hud.PushAlternateHudElement(bid, "resources/hud/sfg_hud.bmp");
	hud.PushAlternateHudElement(bid, "resources/hud/qsfg_hud.bmp");

	// time remaining :
	int timeRemainingHid = hud.PushNewHudElement((SpriteManager::GetResX() - 256) / 2,
												 10,
												 "resources/hud/time_remaining.bmp");
	// final countdown (quand on est mort):
	int finalCountdownId = hud.PushAlternateHudElement(timeRemainingHid, "resources/hud/final_countdown.bmp");
	// time is up:
	int timeIsUpHid = hud.PushAlternateHudElement(timeRemainingHid, "resources/hud/time_is_up.bmp");
	// game over:
	int gameOverHid = hud.PushAlternateHudElement(timeRemainingHid, "resources/hud/game_over.bmp");

	// bar de décompte du temps:
	int timebarHudId = hud.PushProgressBar((SpriteManager::GetResX() - 350) / 2,
										   42,
										   "resources/hud/time_bar_brown.bmp",
										   "resources/hud/time_bar_red.bmp");
	// petite horloge :
	hud.PushAnimatedElement(SpriteManager::GetResX() / 2 + 135,
							14,
							"resources/hud/clock.bmp");

	// accu CP :
	char const* accu_resource = game_vars::g_score_is_acceptable ? "resources/hud/accu_cps_hud.bmp" : "resources/hud/accu_cps_hud_disabled.bmp";
	hud.PushNewHudElement(SpriteManager::GetResX() - 120, 10, accu_resource);

	int cumulScoreHudId = hud.PushNewHudElement(SpriteManager::GetResX() - 138, 64, NULL, 0);

	// compteur de FPS:
	hud.PushNewNumbersFontSprite("resources/hud/numbers_small.bmp");  // id 1

	hud.PushNewHudElement(3, SpriteManager::GetResY() - 40, "resources/hud/fps.bmp");
	int fpshud_elem = hud.PushNewHudElement(5, SpriteManager::GetResY() - 20, NULL, 1);

	// compteur d'ennemis
	hud.PushNewHudElement(40, SpriteManager::GetResY() - 40, "resources/hud/numhostiles.bmp");
	int hostnum_elem = hud.PushNewHudElement(42, SpriteManager::GetResY() - 20, NULL, 1);

	// compteur de kills per minute
	hud.PushNewHudElement(10, 190, "resources/hud/kpm.bmp");
	int kpm_elem = hud.PushNewHudElement(10, 216, NULL, 0);
	// multiplicateur de revenus:
	int cp_times2 = hud.PushNewHudElement(64, 181, "resources/hud/cp_times2.bmp");
	hud.SetElementVisible(cp_times2, false);

	// niveau de vie du joueur
	hud.PushNewHudElement(10, 260, "resources/hud/health_level.bmp");
	int hp_elem = hud.PushNewHudElement(10, 290, NULL, 0);

	// niveau de bouclier du joueur
	int shield_logo = hud.PushNewHudElement(10, 334, "resources/hud/shield_disabled.bmp");
	hud.PushAlternateHudElement(shield_logo, "resources/hud/shield.bmp");
	int shield_val = hud.PushNewHudElement(10, 384, NULL, 0);

	// cadre representant le niveau
	int lvlFrame = sprmngr.LoadSprite("resources/hud/level_overview.bmp", SP_STATIC);
	// sprite representant un pin-point de la position de l'ecran dans le cadre du niveau:
	int positionspr = sprmngr.LoadSprite("resources/hud/position.bmp", SP_STATIC);


// --- creation du shop menu ---

	{
		namespace gshop = game_vars::shop;

		// position :
		int rx225 = (SpriteManager::GetResX() - 450) / 2;
		shop_menu.SetLeftStartPos(rx225 + 32, 240);
		shop_menu.SetRightStartPos(rx225 + 270, 240);
		// en tete:
		shop_menu.SetHeadLeft(rx225, 110, "resources/hud/shop/shop_menu_head_left.bmp");
		shop_menu.SetHeadRight(rx225, 110, "resources/hud/shop/shop_menu_head_right.bmp");
		// elements de gauche (tourelles)
		gshop::g_key_to_id[0].menuid = shop_menu.PushLeft("resources/hud/shop/mg_turret_menu.bmp");
		gshop::g_key_to_id[1].menuid = shop_menu.PushLeft("resources/hud/shop/hmg_tower_menu.bmp");
		gshop::g_key_to_id[2].menuid = shop_menu.PushLeft("resources/hud/shop/rl_tower_menu.bmp");

		// en dessous (les upgrades)
		gshop::g_key_to_id[3].menuid = gshop::g_ccd_id    = shop_menu.PushLeft("resources/hud/shop/ccd_menu.bmp");  // car control device
		gshop::g_key_to_id[4].menuid = gshop::g_shield_id = shop_menu.PushLeft("resources/hud/shop/shield_menu.bmp");  // bouclier

		// elements de droite (armes)
		gshop::g_key_to_id[5].menuid = shop_menu.PushRight("resources/hud/shop/hmg_menu.bmp");
		gshop::g_key_to_id[6].menuid = shop_menu.PushRight("resources/hud/shop/rl_menu.bmp");
		gshop::g_key_to_id[7].menuid = shop_menu.PushRight("resources/hud/shop/sfg_menu.bmp");
		gshop::g_key_to_id[8].menuid = shop_menu.PushRight("resources/hud/shop/qsfg_menu.bmp");
		// curseur:
		shop_menu.SetArrow("resources/hud/arrow.bmp");

		// sprites des touches:
		for (size_t i = 0; i <= 8; ++i)
		{
			std::ostringstream filename;
			filename << "resources/hud/keys/key_" << i << ".bmp";
			gshop::g_key_to_id[i].spriteid = sprmngr.LoadSprite(filename.str().c_str(), SP_STATIC);
		}

	}


// --- initialisation des sprites du car manager et des settings de l'hostile manager : ---

	carmngr.SetExplosionSprite("resources/explode.bmp");
	carmngr.SetWreckageSprite("resources/wreck.bmp");

	enemies.SetSpawnRule(SPR_FREQUENCY | SPR_MIDDLE, infos.enemiesFlow);

	enemies.spriteFiles.push_back("resources/enemy.bmp");
	enemies.spriteFiles.push_back("resources/enemy_strong.bmp");

// --- classes d'hostilités ---

	{
		ia::Manager<3>::StringsRow classes = {"players",
											  "simple hostiles",
											  "strong hostiles"};
		iaManager.SetIAClassNames(classes);

		game_vars::g_playerIAClassID = 0;  // == aussi à iaManager.GetClassID("players"); (voir asserts)
		game_vars::g_simpleIAClassID = 1;
		game_vars::g_strongIAClassID = 2;

		BOOST_ASSERT(iaManager.GetClassID("players")         == game_vars::g_playerIAClassID);
		BOOST_ASSERT(iaManager.GetClassID("simple hostiles") == game_vars::g_simpleIAClassID);
		BOOST_ASSERT(iaManager.GetClassID("strong hostiles") == game_vars::g_strongIAClassID);

		// matrice des relations entre classes d'IA:

		                                  //    players | simpl hos | strong hostiles
		ia::Manager<3>::RelMatrix matrix = { {ia::FRIEND, ia::ENNEMY, ia::ENNEMY},  // players
											 {ia::ENNEMY, ia::FRIEND, ia::FRIEND},  // simple hostiles
											 {ia::ENNEMY, ia::FRIEND, ia::FRIEND}}; // strong hostiles
		iaManager.SetRelations(matrix);

		// quelques verifications
		BOOST_ASSERT(ia::FRIEND == iaManager.GetRelation(1, 2));
		BOOST_ASSERT(ia::ENNEMY == iaManager.GetRelation("players", "simple hostiles"));
	}

	enemies.SetIAClassIDs(game_vars::g_simpleIAClassID, game_vars::g_strongIAClassID);


// --- génération du terrain ---

	terrain.Generate(infos.param);

// -----	  creation du joueur	  -------

	game_vars::g_user = carmngr.Push("resources/car.bmp");
	game_vars::g_user->angle = -1.57f;
	game_vars::g_user->health = 4000;
	game_vars::g_user->shield = 100;
	game_vars::g_user->SetIAClass("players", &iaManager);
	game_vars::g_user->speed = 0.f;
	game_vars::g_user->score = game_vars::g_cheat ? 1000000 : 800;

	// trouver une position de depart correcte pour l'utilisateur.

	{
		int iter = 0;
		bool insertOk = false;
		while (++iter < 10000 && !insertOk)
		{
			game_vars::g_user->SetPosition(80 + rgen.Get(300),
										   100 + rgen.Get(terrain.GetSize().y - 200));
			insertOk = !collider.IntersectRect(game_vars::g_user->rect);
		}
		if (iter == 10000)
		{
			std::cout << "level is too dense" << std::endl;
			return 1;
		}
	}
	// place l'écran au point de départ:
	screen.ForceScreenPosition(Vector2i(game_vars::g_user->rect.pos.x - SpriteManager::GetResX()/2,
										game_vars::g_user->rect.pos.y - SpriteManager::GetResY()/2));

	// enregistrement parmis la liste des hostiles pour être géré comme eux:
	enemies.RegisterPlayer(game_vars::g_user);


// ----	creer des armes dans le gestionnaire :

	// mini gun :
	game_vars::g_minigun_id = arms.CreateWeapon("minigun",
												"resources/weapons/mg_impact.bmp",
												"resources/weapons/mg_flare.bmp",
												"resources/sounds/mg_fire.ogg",
												10,	  // 10 coups par seconde
												8);	  // 8 pts de degats: 100/8=12.5->13 coups pour un ennemi rouge = 1.3s pour le tuer

	// heavy machine gun :
	game_vars::g_heavymg_id = arms.CreateWeapon("hmg",
												"resources/weapons/hm_impact.bmp",
												"resources/weapons/hm_flare.bmp",
												"resources/sounds/machine_gun.ogg",
												7,		  // 7 coups par seconde
												20,	      // 100/20=5 coups -> 1/7*5= 714 ms pour tuer un rouge
												600);	  // elle coute 600 carnage points

	// rocket launcher :
	game_vars::g_rocketl_id = arms.CreateWeapon("rocket launcher",
												"resources/weapons/rl_impact.bmp",
												"resources/weapons/rl_flare.bmp",
												"resources/sounds/rl_fire.ogg",
												2,		  // 2 coups par seconde
												100,	  // 100 pts de dommage a chaque impact direct.
												1900,	  // 1900 cp
												false,
												320,	  // pixels par seconde
												"resources/weapons/rocket.bmp");

	// super fast gun :
	game_vars::g_superfg_id = arms.CreateWeapon("sfg",
												"resources/weapons/hm_impact.bmp",
												"resources/weapons/mg_flare.bmp",
												"resources/sounds/sfg_fire.ogg",
												30,		// 30 coups par seconde
												34,		// 34 pts de degats a chaque impact (100 ms pour tuer)
												5000);	// 5000 carnage points

	// quadruple super fast gun :
	game_vars::g_qsuperfg_id = arms.CreateWeapon("quad sfg",
												 "resources/weapons/hm_impact.bmp",
												 "resources/weapons/mg_flare.bmp",
												 "resources/sounds/sfg_fire.ogg",
												 35,	 // 35 coups par seconde
												 34,	 // 34damage -> 3 coups = 85ms/kill
												 20000,	 // 20000 cp
												 true);	 // quadruple tir

	// on donne le minigun gratos a l'user des le depart :
	game_vars::g_user->GiveWeapon(game_vars::g_minigun_id);
	game_vars::g_currentWeapon = game_vars::g_minigun_id;

// ----- creation des types de tourelles -----

	// minigun turret (id 0)
	towermanager.CreateNewKindOfTower("resources/weapons/mg_turret.bmp",
									  game_vars::g_minigun_id,
									  400);	// 400 carnage points

	// machine gun tower (id 1)
	towermanager.CreateNewKindOfTower("resources/weapons/hm_tower.bmp",
									  game_vars::g_heavymg_id,
									  900);

	// rocket launcher tower (id 2)
	towermanager.CreateNewKindOfTower("resources/weapons/rl_tower.bmp",
									  game_vars::g_rocketl_id,
									  2000);

	// minigun turret - couleurs ennemies ! (id 3)
	towermanager.CreateNewKindOfTower("resources/weapons/mg_turret_enn.bmp",
									  game_vars::g_minigun_id,
									  0);

	// machine gun tower - couleurs ennemies (id 4)
	towermanager.CreateNewKindOfTower("resources/weapons/hm_tower_enn.bmp",
									  game_vars::g_heavymg_id,
									  0);


// --- positionner des tourelles ennemies sur le terrain ---

	enemies.InitGrid();

	smart_ptr< Gunner > towers_owner(new Gunner());
	towers_owner->SetIAClass("simple hostiles", &iaManager);
	{
		int const twrNum = terrain.GetSize().x / 1000;    // grosso-modo: une tourelle ennemie tous les 1000 pixels de large
		int built = 0;
		while (built < twrNum)
		{
			// trouver une position aléatoirement:
			int const x = rgen.Get(terrain.GetSize().x);
			int const y = rgen.Get(terrain.GetSize().y);
			bool farEnough = myutils::dist(x, game_vars::g_user->rect.GetCenter().x) > 400
				|| myutils::dist(y, game_vars::g_user->rect.GetCenter().y) > 400;
			if (farEnough && ChiefManager::GetTowerManager()->BuildNewTower(x, y, rgen.Get(1) + 3, towers_owner.GetPtr()))
				++built;
		}
	}

// --- donner les infos au car manager pourqu'il puisse afficher les positions des voitures sous forme de points dans le cadre du niveau du hud ---

	{
		int redid = sprmngr.LoadSprite("resources/hud/position_en_red.bmp", SP_STATIC);
		int yellowid = sprmngr.LoadSprite("resources/hud/position_en_yellow.bmp", SP_STATIC);
		carmngr.SetCarPositionOverviewInfos(&sprmngr, redid, yellowid, terrain.GetSize());
	}

// ----- chargement des sons et musiques ------

	sounds.PrepareMusicFile("resources/sounds/symheris-winds_of_hope.mp3");
	sounds.StartMusic();
	sounds.SetMusicVolume(infos.musicVolume);

	sounds.PrepareSoundFile("resources/sounds/start.ogg", "start");
	sounds.PrepareSoundFile("resources/sounds/wanna-go-shopping.ogg", "openmenu1");
	sounds.PrepareSoundFile("resources/sounds/2beeps.ogg", "openmenu2");
	sounds.PrepareSoundFile("resources/sounds/that-feels-really-powerful.ogg", "powerful");
	sounds.PrepareSoundFile("resources/sounds/armed.ogg", "loaded");
	sounds.PrepareSoundFile("resources/sounds/wrong.ogg", "beeep");
	sounds.PrepareSoundFile("resources/sounds/repair.ogg", "repair");
	sounds.PrepareSoundFile("resources/sounds/eng-idle.ogg", "engine-idle");
	sounds.PrepareSoundFile("resources/sounds/eng-norm.ogg", "engine-normal");
	sounds.PrepareSoundFile("resources/sounds/eng-fast.ogg", "engine-fast");
	sounds.PrepareSoundFile("resources/sounds/eng-norm_silent.ogg", "engine-muffled");
	sounds.PrepareSoundFile("resources/sounds/player_death.ogg", "death");
	sounds.PrepareSoundFile("resources/sounds/car_explode.ogg", "explosion");
	
	boost::array<const char*, 4> metal_impacts = {{"resources/sounds/metal-hit-1.ogg",
												   "resources/sounds/metal-hit-2.ogg",
												   "resources/sounds/metal-hit-3.ogg",
												   "resources/sounds/metal-hit-4.ogg"}};
	arms.SetCarImpactSounds(metal_impacts);

	KeySummaryScreen();

	sounds.ReadSound("start");
	sounds.ReadSound("engine-idle", 0);

	int frames = 0;
	tools::HiResTimer temps;
	temps.Start();

	float last = 0;
	float current = 0;
	float delta_time = 0;
	float timelimit = 600;  // 10 minutes de jeu

	FPSComputer cfps;
	KPMComputer ckpm;
	MakeDiff<int> differ;

	tools::PerformanceProfiler perfs;

	bool passedTimeZone1 = false;    // zone rouge
	bool passedTimeZone2 = false;    // plus aucun ennemi ne spawn

	bool dead = false;
	float deadtimeout = 30;  // si on meurt, alors on passe en mode "dead timeout count-down"

	SDL_Event event;
	bool exited_with_esckey = false;
	while (!game_vars::gameover && deadtimeout > 0.f)
	{
		// prendre en compte et vider la liste des évenements
		while (SDL_PollEvent(&event))
		{
			HandleEvent(event);
		}

		if (game_vars::gameover)  // menu pause: premier niveau de sureté avant de quitter
		{
			SetEngineSound(SES_OFF);
			temps.Stop();

			EPauseMenuChoice pmc = PauseMenu();
			if (pmc == PMC_CONTINUE)
				game_vars::gameover = false;
			else if (pmc == PMC_QUIT)
				exited_with_esckey = true;  // le joueur veut sortir

			temps.Start();
			SetEngineSound(SES_ON);
		}

		Vector2i attractCenter(game_vars::g_user->rect.GetCenter());
		attractCenter.x += (int)(game_vars::g_user->direction.x * 180.f);
		attractCenter.y += (int)(game_vars::g_user->direction.y * 180.f);
		screen.MoveMagnetCenter(attractCenter);
		screen.Update(delta_time);

		perfs.BeginProfile("hostiles", temps.GetTime());
		enemies.Update(delta_time, attractCenter, (current / timelimit > 0.5f) ? true : false);   // a partir de la moitié du temps, les ennemis deviennent jaune
		perfs.EndProfile("hostiles", temps.GetTime());

		SpriteManager::scroll_x = screen.GetRc().pos.x;
		SpriteManager::scroll_y = screen.GetRc().pos.y;

		perfs.BeginProfile("draw terrain", temps.GetTime());
		terrain.DrawBackGround();
		perfs.EndProfile("draw terrain", temps.GetTime());

		sprmngr.Draw(lvlFrame, 200, 8);  // par dessus le terrain mais sous les voitures (car elles dessinent leurs 'dots')

		perfs.BeginProfile("draw cars", temps.GetTime());
		carmngr.DrawCars();
		perfs.EndProfile("draw cars", temps.GetTime());

		perfs.BeginProfile("draw ground objects", temps.GetTime());
		terrain.DrawHighGround();
		perfs.EndProfile("draw ground objects", temps.GetTime());

		perfs.BeginProfile("tower update", temps.GetTime());
		towermanager.Update(current);
		perfs.EndProfile("tower update", temps.GetTime());

		perfs.BeginProfile("firearms update", temps.GetTime());
		arms.Update(current);
		perfs.EndProfile("firearms update", temps.GetTime());

		if (!dead && game_vars::g_user->health <= 0)
		{
			game_vars::g_user->health = 0;
			SetEngineSound(SES_OFF);
			sounds.ReadSound("death");
			dead = true;
		}

		// mettre a jour les valeurs du Head Up Display:

		hud.UpdateNumberValue(scoreHudId, game_vars::g_user->score);
		hud.UpdateNumberValue(cumulScoreHudId, game_vars::g_user->score_cumul);
		hud.UpdateNumberValue(fpshud_elem, (int)cfps.GetLastAverage());
		hud.UpdateNumberValue(hostnum_elem, enemies.GetEnnemiesNum());
		hud.UpdateNumberValue(hp_elem, game_vars::g_user->health);

		{
			// afficher le bouclier tramé ou le plein:
			bool hasshield = game_vars::g_user->shield > 0;
			hud.UpdateAlternateElement(shield_logo, hasshield ? 1 : 0);
			hud.SetElementVisible(shield_val, hasshield);  // on affiche les nombres ou pas
			// mise a jour de la valeur:
			hud.UpdateNumberValue(shield_val, game_vars::g_user->shield);
		}

		// compteur de kill per minute:
		size_t kpm = ckpm.RegisterKill(differ.GetDiff(game_vars::g_user->GetKillCount()), current);

		// multiplicateur de score en fonction des KPM:
		if (kpm < 50)
			game_vars::g_user->income_multiplier = 1;
		else if (kpm >= 50)
			game_vars::g_user->income_multiplier = 2;

		hud.SetElementVisible(cp_times2, game_vars::g_user->income_multiplier == 2);

		hud.UpdateNumberValue(kpm_elem, kpm);
		hud.UpdateAlternateElement(bid, game_vars::g_currentWeapon);

		// time bar et espace associé:
		{
			bool gameIsOver = passedTimeZone2 && enemies.GetEnnemiesNum() == 0;
			bool timeIsUp = passedTimeZone2;
			int idOfSpriteToPrintAboveTimebar = gameIsOver ?  gameOverHid  :  (timeIsUp ? timeIsUpHid : 0);
			hud.UpdateAlternateElement(timeRemainingHid, idOfSpriteToPrintAboveTimebar);
			int value = 0;
			if (!dead)
				value = (int)((timelimit - current) * 100.f / timelimit);
			else
			{
				static float maxdeadtimeout = deadtimeout;
				value = (int)((deadtimeout / maxdeadtimeout) * 100.f);
				deadtimeout -= delta_time;
				bool print_FinalCountdown = ((int)(deadtimeout) % 2) != 0;
				if (print_FinalCountdown)
					hud.UpdateAlternateElement(timeRemainingHid, finalCountdownId);
			}
			hud.UpdateProgressBar(timebarHudId, value);
		}

		if ((current / timelimit) > 0.8f && !passedTimeZone1)
		{  // on entre dans la zone rouge
			enemies.SetSpawnRule(SPR_FREQUENCY | SPR_RIGHT, infos.enemiesFlow * 7);
			passedTimeZone1 = true;
		}
		if (current > timelimit && !passedTimeZone2)
		{
			enemies.SetSpawnRule(SPR_DISABLED);  // fini !
			passedTimeZone2 = true;
		}

		perfs.BeginProfile("hud draw", temps.GetTime());
		hud.Draw();
		perfs.EndProfile("hud draw", temps.GetTime());

		if (game_vars::shop::g_menuActivated)
		{
			DrawShopMenu(shop_menu, sprmngr);
		}

		// dessin de la position de l'ecran dans le niveau:
		sprmngr.Draw(positionspr, ((float)(SpriteManager::scroll_x) / terrain.GetSize().x)*80 + 200,
					              ((float)(SpriteManager::scroll_y) / terrain.GetSize().y)*40 + 8);

		terrain.Update(current);
		carmngr.Update(current);
		sprmngr.UpdateWithClientTime(current);
		sounds.Update(game_vars::g_user->rect.GetCenter());

		SetPassingCarSound();

		++frames;

		current = temps.GetTime();
		delta_time = current - last;
		last = current;

		cfps.ComputeFPS(delta_time, current);
	}

	SetEngineSound(SES_OFF);

	// ecran de calcul du score
	{
		game_vars::g_user->score_cumul =
			FinalScoreMenu(game_vars::g_user->score, game_vars::g_user->score_cumul, exited_with_esckey && !passedTimeZone2);
	}

	if (game_vars::g_score_is_acceptable)
	{
		// on ecrit le score:
		ChiefManager::GetHighScoreManager()->AddScore(GetPlayerFromIndex(infos.pickedPlayer), game_vars::g_user->score_cumul);
		ChiefManager::GetHighScoreManager()->WriteOnFile();
	}

	sounds.StopMusic();
	sounds.StopSound("engine-normal");
	sounds.StopSound("engine-idle");
	sounds.StopSound("engine-fast");

#if defined(_DEBUG)
	std::cout << frames / temps.GetTime() << " fps average" << std::endl << temps.GetTime() << " seconds" << std::endl << std::endl;

	std::cout << perfs << std::endl;
#endif  // _DEBUG

	ChiefManager::ResetAll();

	return 0;  // destruction des managers ici.
}

// safety kill thread
void* WaitAndTerminate(void*)
{
	sleep(5);
	std::cout << "warn: couldn't close audio after 5 seconds : kill application" << std::endl;
	exit(0);
	return 0;
}

int OnlyAcceptKeyboardEvents(const SDL_Event* e)
{
	switch (e->type)
	{
		case SDL_QUIT:
		case SDL_KEYDOWN:
		case SDL_KEYUP:
			return 1;
		default:
			break;
	}
	return 0;
}

int main(int argc, char* argv[])
{
	if (argc == 2 && !strcmp(argv[1], "--help"))
	{
		std::cout << "you can enter option:" << std::endl;
		std::cout << " --cheat to activate developer mode" << std::endl;
		std::cout << "to change screen resolution, edit first two lines of " CONF_FILE_NAME " file." << std::endl;
		return 0;
	}

	// on peut activer la triche en ligne de commande :
	if (argc >= 2)
	{
		if (!strcmp(argv[1], "--cheat"))
			game_vars::g_cheat = true;
	}

	int winResX, winResY, musicVolume;
	ReadConfFile(&winResX, &winResY, &musicVolume);

	CustomTerrainParams custom_params;

	bool quitGame = false;
	bool back_from_party = false;
	SoundSettings sndSet;
	sndSet.musicVolume = musicVolume;
	while (!quitGame)
	{
		// afficher le menu d'accueil

		int pickedPlayer = -1;

		// paramètres standards pour partie classique (les seuls qui donnent accès aux highscores) :
		custom_params.density[KTO_WATER] = 0.03;
		custom_params.density[KTO_TREE] = 0.012;
		custom_params.density[KTO_ROCKS] = 0.02;
		custom_params.size_long = 80;
		custom_params.flow = 300;
		custom_params.seed = 100;

		if (game_vars::g_cheat)
		{
			game_vars::g_score_is_acceptable = false;
		}

		{
			SoundManager menuMusic(SoundManager::CP_DONT_CLOSE);
			menuMusic.PrepareMusicFile("resources/sounds/entrance.mp3");
			menuMusic.SetMusicVolume(sndSet.musicVolume);
			menuMusic.StartMusic();
			menuMusic.PrepareSoundFile("resources/sounds/tit.ogg", "tit");

			render2d::SharedSurface menuSurface(800, 600, render2d::DP_WEAK);
			SDL_SetEventFilter(OnlyAcceptKeyboardEvents);

			EChoosePlayerMenuCommands ePickedPlayer = CPMC_RETURN;
			// tant que pas choisi un joueur ou qu'on veut pas quitter
			while (ePickedPlayer == CPMC_RETURN && !quitGame)
			{
				EMainMenuCommands command = back_from_party ? MC_HIGHSCORES : MC_NONE;
				while (command != MC_START && !quitGame)
				{
					if (!back_from_party)
						command = WelcomeMenu();

					menuMusic.ReadSound("tit");

					back_from_party = false;
					if (command == MC_CUSTOMIZED_TERRAIN)
					{
						std::pair< CustomTerrainParams, bool > ret = CustomizeTerrainMenu(custom_params);
						if (ret.second)  // si le joueur a fait entrée, sinon on touche a rien
						{
							custom_params = ret.first;
							game_vars::g_score_is_acceptable = false;
							command = MC_START;
						}
					}
					else if (command == MC_HIGHSCORES)
					{
						HighScoresMenu();
					}
					else if (command == MC_SOUND_SETTINGS)
					{
						SoundSettings ret = SoundSettingsMenu(sndSet.musicVolume, &menuMusic);
						if (ret.musicVolume >= 0)
							sndSet = ret;  // si le joueur a validé
					}
					else if (command == MC_EXITGAME)
					{
						DisplayCreditsScreen();
						quitGame = true;
					}
					menuMusic.ReadSound("tit");
				};
				if (!quitGame)
				{
					// on arrive ici quand l'utilisateur a demandé le départ
					ePickedPlayer = game_vars::g_score_is_acceptable ? ChoosePlayerMenu() : CPMC_BAD_GUY;
					switch (ePickedPlayer)
					{
						case CPMC_BAD_GUY:
							pickedPlayer = 0;
							break;
						case CPMC_MORPH:
							pickedPlayer = 1;
							break;
						default:
							break;
					}
					menuMusic.ReadSound("tit");
				}
			}
			// fin des menus, lancement du jeu
			menuMusic.StopMusic();
		} // destruction de la surface partagée

		if (!quitGame)
		{
			GameInfos infos;
			infos.pickedPlayer = pickedPlayer;
			infos.resx = winResX;
			infos.resy = winResY;
			infos.musicVolume = sndSet.musicVolume;
			infos.param.seed = custom_params.seed;
			infos.enemiesFlow = custom_params.flow;

// ---- réglage des paramètres pour la génération du terrain

			infos.param.size_long = custom_params.size_long;
			infos.param.size_height = infos.param.size_long / 2;

			infos.param.density[KTO_WATER] = custom_params.density[KTO_WATER];
			infos.param.density[KTO_TREE] = custom_params.density[KTO_TREE];
			infos.param.density[KTO_ROCKS] = custom_params.density[KTO_ROCKS];

			infos.param.terrain_sprite = "resources/terrain/terrain.bmp";
			infos.param.wall_sprite = "resources/terrain/wall.bmp";
			infos.param.finalline_sprite = "resources/terrain/final_line.bmp";

			infos.param.sprites[KTO_WATER].push_back("resources/terrain/water.bmp");
			infos.param.sprites[KTO_WATER].push_back("resources/terrain/water2.bmp");

			infos.param.sprites[KTO_TREE].push_back("resources/terrain/tree.bmp");
			infos.param.sprites[KTO_TREE].push_back("resources/terrain/tree2.bmp");
			infos.param.sprites[KTO_TREE].push_back("resources/terrain/tree3.bmp");

			infos.param.sprites[KTO_ROCKS].push_back("resources/terrain/rocks.bmp");
			infos.param.sprites[KTO_ROCKS].push_back("resources/terrain/rocks2.bmp");

			StartGame(infos);
		}

		if (game_vars::g_score_is_acceptable)
			back_from_party = true;
		game_vars::Reset();
	}

	myutils::MakeThread(&WaitAndTerminate);  // SDL bloque souvent sur l'arret de l'audio, ptit override, si ca marche pas => boum apres un temps.

	SoundManager::CloseSDLMIXAudio();
	render2d::SDLLife::ManuallyQuitSDL();
	return 0;
}
