/**********************************************************************
 * File  : csttimegetsizelist.hpp
 * Author: Lightness1024
 *         mailto:lightness1024@free.fr
 * Date  : 09/05/2009
 *********************************************************************/

#ifndef CSTTIMEGETSIZELIST_HPP_INC_09_05_2009__769066_
#define CSTTIMEGETSIZELIST_HPP_INC_09_05_2009__769066_ 1

namespace tools
{
	template< typename T >
	struct CstTimeGetSizeList : private std::list< T >  // on se la met en privé pour eviter de rater des redefinitions importantes
	{
		typedef typename std::list<T>::iterator iterator;
		typedef typename std::list<T>::value_type value_type;
		typedef typename std::list<T>::reference reference;

		iterator erase(iterator first, iterator last)
		{
			size_ -= std::distance(first, last);
			return std::list<T>::erase(first, last);
		}
		iterator erase(iterator position)
		{
			--size_;
			return std::list<T>::erase(position);
		}
		void push_front(value_type const& x)
		{
			++size_;
			std::list<T>::push_front(x);
		}
		void push_back(value_type const& x)
		{
			++size_;
			std::list<T>::push_back(x);
		}
		void pop_back()
		{
			--size_;
			std::list<T>::pop_back();
		}
		void pop_front()
		{
			--size_;
			std::list<T>::pop_front();
		}
		size_t size()
		{
			return size_;
		}
		void clear()
		{
			size_ = 0;
			std::list<T>::clear();
		}
		
		reference front()
		{
			return std::list<T>::front();
		}
		iterator begin()
		{
			return std::list<T>::begin();
		}
		iterator end()
		{
			return std::list<T>::end();
		}

		CstTimeGetSizeList() : size_(0)
		{}

	private:
		size_t size_;  // garder la taille de la liste dans un coin, parce que std::list ne peut pas la donner en temps constant !
	};
}

#endif //CSTTIMEGETSIZELIST_HPP_INC_09_05_2009__769066_
