/**********************************************************************
 * File	: weaponmanager.hpp
 * Author: Vivien - a.k.a. Lightness1024!
 *			  mailto:lightness1024@free.fr
 * Date	: 15/07/2007
 *********************************************************************/

#ifndef FPSCOMPUTER_HPP_INC_19_03_2009__225442723_
#define FPSCOMPUTER_HPP_INC_19_03_2009__225442723_ 1

#include <list>
#include <map>
#include <string>
#include <algorithm>
#include <iostream>
#include "smart_ptr.hpp"

#define CAPTURED_FRAMES_NUM 30    // 30 captures
#define AVG_TIME            0.5   // 500 millisecondes

// calcul des frames per second
class FPSComputer
{
   double history[CAPTURED_FRAMES_NUM];  // fera office de file circulaire
	int indx;
	double timestamp;
	const double step;
	double average;
	double last;

public:

	FPSComputer(void)
		: indx(0),
		  timestamp(0),
		  step(AVG_TIME / CAPTURED_FRAMES_NUM),
		  average(0),
		  last(0)
	{
		for (int i = 0; i < CAPTURED_FRAMES_NUM; ++i)
			history[i] = 0;
	}

	double ComputeFPS(double delta_time, double total_time)
	{
		double fpsFrame = 1. / delta_time;
		if (total_time - last > step)      // capture a frquence limite
		{
			last = total_time;
			++indx %= CAPTURED_FRAMES_NUM;  // incrmentation circulaire
   		// mise  jour de la moyenne
			average -= history[indx];       // queue de la file
			history[indx] = fpsFrame / CAPTURED_FRAMES_NUM;
			average += history[indx];       // tte de la file (maintenant)
		}
		return average;
	}

	double GetLastAverage(void) { return average; }
};

// calcul des kill per minute
class KPMComputer
{
	float kpm;
	std::list< float > killstamps;

public:

	KPMComputer(void)
		: kpm(0)
	{ }

	// *** mthodes ***
	
	size_t RegisterKill(int numkills, float total_time)
	{
		// on enregistre des horodatage de kills
		for (int nk = 0; nk < numkills; ++nk)
			killstamps.push_back(total_time);

		// on enlve les lments du dbut de la liste, quand les kills sont plus vieux qu'une minute
		while (killstamps.size() && killstamps.front() <= total_time - 60)
			killstamps.pop_front();

		return killstamps.size();
	}
};


#endif //FPSCOMPUTER_HPP_INC_19_03_2009__225442723_
