/**********************************************************************
 * File  : timer.hpp
 * Author: Lightness1024
 *         mailto:lightness1024@free.fr
 * Date  : 19/03/2009
     multiplatform timer
     using processor internal performance registers
     nanosecond precision on Mac and Win, microsecond on linux.
 *********************************************************************/

#ifndef TIMER_HPP_INC_19_03_2009__28917117_
#define TIMER_HPP_INC_19_03_2009__28917117_ 1

#if defined( MACOS )
 #include "tl_mach_osstime.hpp"
#elif defined( __linux )
 #include  "tl_posix_osstime.hpp"
#else
 #include "tl_win_osstime.hpp"
#endif

namespace tools
{
	//! StopWatch class with nanosecond precision
	class HiResTimer
	{
	public:
		enum ERunMode
		{
			RM_STARTED,
			RM_STOPPED
		};
		inline void Reset();                        //!< reset the stopwatch to 00:00:00
		inline double GetTime() const;              //!< read current time without affecting stopwatch state
		inline double Stop();                       //!< freeze the clock on the current time
		inline void Start();                        //!< start from the current time
		static inline void Sleep(int milliseconds); //!< pause the thread for a specified amount of time

		inline HiResTimer();                        //!< construct a stopped and reseted stopwatch
		inline HiResTimer(ERunMode startMode);      //!< construct a reseted stopwatch, with parametrable running mode
		
	private:
		OSSpecTime_t start_;
		OSSpecTime_t total_;
		bool bRun_;
	};
}

#include "timer.inl"

#endif //TIMER_HPP_INC_19_03_2009__28917117_
