/**********************************************************************
 * File  : tl_posix_osstime.h
 * Author: Lightness1024
 *         mailto:lightness1024@free.fr
 * Date  : 19/03/2009
 *********************************************************************/

#ifndef TL_POSIX_OSSTIME_H_INC_19_03_2009___21084060_
#define TL_POSIX_OSSTIME_H_INC_19_03_2009___21084060_ 1

#include <sys/time.h>
#include <time.h>
#include <unistd.h>

// POSIX microsecond implementation
namespace tools
{
	struct OSSpecTime_t
	{
		OSSpecTime_t operator - (OSSpecTime_t const & right) const
		{
			OSSpecTime_t t;
			t.val_.tv_sec = val_.tv_sec - right.val_.tv_sec;
			t.val_.tv_usec = val_.tv_usec - right.val_.tv_usec;
			if (t.val_.tv_usec < 0)
			{
				t.val_.tv_usec += 1e6;
				t.val_.tv_sec -= 1;
			}
			return t;
		}
		OSSpecTime_t& operator += (OSSpecTime_t const & right)
		{
			val_.tv_sec += right.val_.tv_sec;
			val_.tv_usec += right.val_.tv_usec;
			if (val_.tv_usec > 1e6)
			{
				val_.tv_usec -= 1e6;
				val_.tv_sec += 1;
			}
			return *this;
		}
		timeval val_;
	};
	inline OSSpecTime_t GetSysTime()
	{
		OSSpecTime_t t;
		gettimeofday(&t.val_, 0);
		return t;
	}
	inline double ToSeconds(OSSpecTime_t const & osst_t)
	{
		return osst_t.val_.tv_sec + 1.0e-6 * osst_t.val_.tv_usec;
	}
	inline void MilliSleep(int ms)
	{
		usleep(ms * 1000);
	}
}


#endif //TL_POSIX_OSSTIME_H_INC_19_03_2009___21084060_
